﻿// Circle Dock 0.9.2 - Copyright 2008 Eric Wong
// Circle Dock is open source software licensed under GNU GENERAL PUBLIC LICENSE V3. See included Licence.txt file.
// http://circledock.wikidot.com
// VideoInPicture@gmail.com

// This file a class that allows for fast bitmap processing on a per pixel basis. It is much faster than the default .Net method.
// For PixelFormat.Format32bppArgb bitmaps.
// Origin/Original Author is unknown.
// Modified by Eric Wong to support alpha channels and commented by Eric Wong.

// Example Use
//    FastBitmap image = new FastBitmap(SomeBitmap, false);
//    image.LockPixels();
//    byte[] pixels = image.GetAllPixels();

//    for (int i = 0; i < pixels.Length; i++)
//    {
//        pixels[i] = (byte)(255 - pixels[i]);
//    }

//    image.SetAllPixels(pixels);
//    image.UnlockPixels();


using System;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;


namespace FastBitmapProcessing
{
    /// <summary>    
    /// Allows fast manipulation of bitmaps on a per pixel basis. For PixelFormat.Format32bppArgb bitmaps.
    /// </summary>   
    public class FastBitmap
    {
        #region Variables

        private Bitmap image;

        private BitmapData bitmapData;

        private int height;

        private int width;

        private int stride;

        private byte[] rgbValues;

        private bool locked = false;

        #endregion

        #region Constructors

        /// <summary>        
        /// Creates a new instance of FastBitmap and initializes it with a new bitmap of size x by y.
        /// </summary>        
        /// <param name="x">Width of the desired bitmap.</param>  
        /// <param name="y">Height of the desired bitmap.</param>  
        public FastBitmap(int x, int y)
        {
            width = x;
            height = y;
            image = new Bitmap(x, y);
        }

        /// <summary>        
        /// Creates a new instance of FastBitmap and initializes it with a given bitmap.
        /// </summary>        
        /// <param name="bitmap">Desired bitmap to manipulate.</param>  
        /// <param name="createCopy">True to make a copy of the bitmap so the original is unmodified.</param>  
        public FastBitmap(Bitmap bitmap, bool createCopy)
        {
            width = bitmap.Width;
            height = bitmap.Height;

            if (createCopy == true)
                image = new Bitmap(bitmap);
            else
                image = bitmap;
        }

        #endregion

        #region Methods

        /// <summary>        
        /// Gets the height of the bitmap.
        /// </summary>   
        public int Height
        {
            get { return this.height; }
        }

        /// <summary>        
        /// Gets the width of the bitmap.
        /// </summary> 
        public int Width
        {
            get { return this.width; }
        }

        /// <summary>        
        /// Gets all the ARGB values for the pixels in the bitmap in a byte array.
        /// </summary>        
        /// <returns>A byte array representing the ARGB values of the pixels in the bitmap with a size of bitmap width x height.</returns> 
        public byte[] GetAllPixels()
        {
            return rgbValues;
        }

        /// <summary>        
        /// Sets the ARGB values for all the pixels in the bitmap.
        /// </summary>        
        /// <param name="pixels">A byte array with the ARGB values with a size of bitmap width x height.</param>     
        /// <returns>Nothing</returns> 
        public void SetAllPixels(byte[] pixels)
        {
            rgbValues = pixels;
        }

        /// <summary>        
        /// Gets the ARGB values for a pixel in a Color.
        /// </summary>        
        /// <param name="x">X-position of the pixel. Must be within bounds of the bitmap.</param>    
        /// <param name="y">Y-position of the pixel. Must be within bounds of the bitmap.</param> 
        /// <returns>A Color representing the ARGB values for the pixel.</returns> 
        public Color GetPixel(int x, int y)
        {
            int s = stride;

            int blue = rgbValues[(y * s + (x * 4))];
            int green = rgbValues[(y * s + (x * 4)) + 1];
            int red = rgbValues[(y * s + (x * 4)) + 2];
            int alpha = rgbValues[(y * s + (x * 4)) + 3];

            return Color.FromArgb(alpha, red, green, blue);
        }

        /// <summary>        
        /// Gets the ARGB values for a pixel in a 4-byte array.
        /// </summary>        
        /// <param name="x">X-position of the pixel. Must be within bounds of the bitmap.</param>    
        /// <param name="y">Y-position of the pixel. Must be within bounds of the bitmap.</param> 
        /// <returns>A 4-byte array representing the ARGB values for the pixel.</returns> 
        public byte[] GetPixelValues(int x, int y)
        {
            int w = width;
            int s = stride;
            byte[] c = new byte[4];

            c[3] = rgbValues[(y * s + (x * 4))];
            c[2] = rgbValues[(y * s + (x * 4)) + 1];
            c[1] = rgbValues[(y * s + (x * 4)) + 2];
            c[0] = rgbValues[(y * s + (x * 4)) + 3];

            return c;
        }

        /// <summary>        
        /// Set a pixel value of the bitmap.
        /// </summary>        
        /// <param name="x">X-position of the pixel. Must be within bounds of the bitmap.</param>    
        /// <param name="y">Y-position of the pixel. Must be within bounds of the bitmap.</param> 
        /// <param name="cIn">The Color desired for the pixel.</param> 
        /// <returns>Nothing</returns> 
        public void SetPixel(int x, int y, Color cIn)
        {
            int s = stride;

            rgbValues[(y * s + (x * 4))] = cIn.B;
            rgbValues[(y * s + (x * 4)) + 1] = cIn.G;
            rgbValues[(y * s + (x * 4)) + 2] = cIn.R;
            rgbValues[(y * s + (x * 4)) + 3] = cIn.A;
        }

        /// <summary>        
        /// Set a pixel value of the bitmap.
        /// </summary>        
        /// <param name="x">X-position of the pixel. Must be within bounds of the bitmap.</param>    
        /// <param name="y">Y-position of the pixel. Must be within bounds of the bitmap.</param> 
        /// <param name="a">Alpha pixel value.</param> 
        /// <param name="r">Red pixel value.</param> 
        /// <param name="g">Green pixel value.</param>
        /// <param name="b">Blue pixel value.</param> 
        /// <returns>Nothing</returns> 
        public void SetPixel(int x, int y, byte a, byte r, byte g, byte b)
        {
            int w = width;
            int s = stride;

            rgbValues[(y * s + (x * 4))] = b;
            rgbValues[(y * s + (x * 4)) + 1] = g;
            rgbValues[(y * s + (x * 4)) + 2] = r;
            rgbValues[(y * s + (x * 4)) + 3] = a;
        }

        /// <summary>        
        /// Allows typecasting of a FastBitmap class to an Image.
        /// </summary>           
        /// <param name="bmp">An instance of the FastBitmap class.</param> 
        /// <returns>The image stored for the instance of bmp.</returns> 
        public static implicit operator Image(FastBitmap bmp)
        {
            return bmp.image;
        }

        /// <summary>        
        /// Allows typecasting of a FastBitmap class to a Bitmap.
        /// </summary>           
        /// <param name="bmp">An instance of the FastBitmap class.</param> 
        /// <returns>The image stored for the instance of bmp.</returns> 
        public static implicit operator Bitmap(FastBitmap bmp)
        {
            return bmp.image;
        }

        /// <summary>        
        /// Locks the bitmap data in memory to allow for faster access.
        /// </summary>               
        /// <returns>Nothing</returns> 
        public void LockBitmap()
        {
            LockBitmap(new Rectangle(0, 0, width, height));
        }

        /// <summary>        
        /// Helper method that locks the bitmap data in memory to allow for faster access.
        /// </summary>        
        /// <param name="area">The area of the bitmap to lock.</param>        
        /// <returns>Nothing</returns> 
        private void LockBitmap(Rectangle area)
        {
            if (locked) return;

            locked = true;

            bitmapData = image.LockBits(area, ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);

            stride = bitmapData.Stride;

            IntPtr ptr = bitmapData.Scan0;

            int numBytes = bitmapData.Stride * height;

            rgbValues = new byte[numBytes];

            Marshal.Copy(ptr, rgbValues, 0, numBytes);
        }

        /// <summary>        
        /// Unlocks the bitmap data in memory to allow it to be used by others.
        /// Must call this method when you are finished manipulating the bitmap or 
        /// errors may be thrown by other methods.
        /// </summary>               
        /// <returns>Nothing</returns> 
        public void UnlockBitmap()
        {
            if (!locked) return;

            locked = false;

            Marshal.Copy(rgbValues, 0, bitmapData.Scan0, bitmapData.Stride * image.Height);

            image.UnlockBits(bitmapData);
        }

        #endregion
    }

}


