﻿// Circle Dock 0.9.2 - Copyright 2008 Eric Wong
// Circle Dock is open source software licensed under GNU GENERAL PUBLIC LICENSE V3. See included Licence.txt file.
// http://circledock.wikidot.com
// VideoInPicture@gmail.com

// This file contains the class used to load the .ini language file from a given path. Depends on LanguageInformation.cs.


using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;
using System.IO;

using AMS.Profile;      // Allows for .ini file manipulation
using LanguageInformation;  

namespace LanguageLoader
{
    public class LanguageLoader
    {
        #region Variables

        private Profile LanguageINI;

        public LanguageInformation.General General;
        public LanguageInformation.LanguageFile LanguageFile;
        public LanguageInformation.MainContextMenu MainContextMenu;
        public LanguageInformation.SettingsPanel SettingsPanel;
        public LanguageInformation.ErrorMessages ErrorMessages;
        public LanguageInformation.DockItemProperties DockItemProperties;

        #endregion

        #region Constructor & Private Methods

        /// <summary>    
        /// Loads the language information from the given FilePath.
        /// The language data is accessed by going to the public structs in this class.
        /// The structs will return "?????" if it cannot find valid language data.
        /// </summary>  
        public LanguageLoader(String FilePath)
        {
            if (!File.Exists(FilePath))
            {
                MessageBox.Show(@"Language file: " + FilePath + @" is missing." + "\r" + @"Please replace the language file.", "Circle Dock");
            }

            LanguageINI = new Ini(FilePath);

            InitializeLanguageInfoStructs();

            LoadLanguageData();
        }

        public void InitializeLanguageInfoStructs()
        {
            General = new LanguageInformation.General();
            LanguageFile = new LanguageInformation.LanguageFile();
            MainContextMenu = new LanguageInformation.MainContextMenu();
            SettingsPanel = new LanguageInformation.SettingsPanel();
            ErrorMessages = new LanguageInformation.ErrorMessages();
        }

        public void LoadLanguageData()
        {
            LoadGeneral();
            LoadLanguageFile();
            LoadMainContextMenu();
            LoadSettingsPanel();
            LoadErrorMessages();
            LoadDockItemProperties();
        }

        public void LoadLanguageDataFromNewFile(String FilePath)
        {
            LanguageINI = new Ini(FilePath);
            LoadLanguageData();
        }

        private String GetEntry(String Section, String EntryName)
        {
            //Console.WriteLine("here language: " + Section + "  " + EntryName);
            String languageEntry = null;

            try
            {
                languageEntry = (String)LanguageINI.GetValue(Section, EntryName);
            }
            catch (Exception)
            { }

            if (languageEntry != null)
                return languageEntry;
            else
            {
                try
                {
                    LanguageINI.SetValue(Section, EntryName, @"?????");
                }
                catch (Exception)
                { }
                return @"?????";
            }
        }

        #endregion

        #region Load Each Section for LanguageInformation

        private void LoadGeneral()
        {
            General.AuthorWord = GetEntry("General", "AuthorWord");
            General.CircleDockName = GetEntry("General", "CircleDockName");
            General.GeneralWebsite = GetEntry("General", "GeneralWebsite");
            General.HelpWebsite = GetEntry("General", "HelpWebsite");
            General.CircleDockWebsiteWord = GetEntry("General", "CircleDockWebsiteWord");
            General.SupportForumWord = GetEntry("General", "SupportForumWord");
            General.SupportForumWebsite = GetEntry("General", "SupportForumWebsite");
            General.RotationSensitivity = GetEntry("General", "RotationSensitivity");
            General.IconReplacementMode = GetEntry("General", "IconReplacementMode");

            General.normal = GetEntry("General", "normal");
            General.topmost = GetEntry("General", "topmost");
            General.alwaysOnBottom = GetEntry("General", "alwaysOnBottom");
            General.zLevel = GetEntry("General", "zLevel");

        }

        private void LoadLanguageFile()
        {
            LanguageFile.CircleDockVersion = GetEntry("LanguageFile", "CircleDockVersion");
            LanguageFile.LanguageName = GetEntry("LanguageFile", "LanguageName");
            LanguageFile.LanguageFileVersion = GetEntry("LanguageFile", "LanguageFileVersion");
            LanguageFile.FileAuthor = GetEntry("LanguageFile", "FileAuthor");
        }

        private void LoadMainContextMenu()
        {
            MainContextMenu.HideWord = GetEntry("MainContextMenu", "HideWord");
            MainContextMenu.ShowWord = GetEntry("MainContextMenu", "ShowWord");
            MainContextMenu.QuitWord = GetEntry("MainContextMenu", "QuitWord");
            MainContextMenu.SettingsWord = GetEntry("MainContextMenu", "SettingsWord");
            MainContextMenu.WebsiteWord = GetEntry("MainContextMenu", "WebsiteWord");
            MainContextMenu.RemoveWord = GetEntry("MainContextMenu", "RemoveWord");
            MainContextMenu.AddWord = GetEntry("MainContextMenu", "AddWord");
            MainContextMenu.DockFolder = GetEntry("MainContextMenu", "DockFolder");
            MainContextMenu.ChangeIcon = GetEntry("MainContextMenu", "ChangeIcon");
            MainContextMenu.ItemSettings = GetEntry("MainContextMenu", "ItemSettings");
            MainContextMenu.PauseMouseToggling = GetEntry("MainContextMenu", "PauseMouseToggling");
            MainContextMenu.blankIcon = GetEntry("MainContextMenu", "blankIcon");
        }

        private void LoadSettingsPanel()
        {
            SettingsPanel.Name = GetEntry("SettingsPanel", "Name");
            SettingsPanel.SettingsPanelWord = GetEntry("SettingsPanel", "SettingsPanelWord");
            SettingsPanel.General = GetEntry("SettingsPanel", "GeneralWord");

            SettingsPanel.DockShape = GetEntry("SettingsPanel", "DockShape");
            
            SettingsPanel.Labels = GetEntry("SettingsPanel", "Labels");
            SettingsPanel.Animation = GetEntry("SettingsPanel", "Animation");
            SettingsPanel.LocationWord = GetEntry("SettingsPanel", "LocationWord");
            SettingsPanel.TogglingWord = GetEntry("SettingsPanel", "TogglingWord");
            SettingsPanel.FileDockIconAssociations = GetEntry("SettingsPanel", "FileDockIconAssociations");
            SettingsPanel.LanguageWord = GetEntry("SettingsPanel", "LanguageWord");
            SettingsPanel.HelpWord = GetEntry("SettingsPanel", "HelpWord");
            SettingsPanel.CircleDockVersionWord = GetEntry("SettingsPanel", "CircleDockVersionWord");
            SettingsPanel.EnableDockRotation = GetEntry("SettingsPanel", "EnableDockRotation");
            SettingsPanel.HotKeySelectorToggleDock = GetEntry("SettingsPanel", "HotKeySelectorToggleDock");
            SettingsPanel.CtrlWordAbbreviation = GetEntry("SettingsPanel", "CtrlWordAbbreviation");
            SettingsPanel.AltWordAbbreviation = GetEntry("SettingsPanel", "AltWordAbbreviation");
            SettingsPanel.ShiftWordAbbreviation = GetEntry("SettingsPanel", "ShiftWordAbbreviation");
            SettingsPanel.WindowsKeyWordAbbreviation = GetEntry("SettingsPanel", "WindowsKeyWordAbbreviation");
            SettingsPanel.ValidHotkeyCombination = GetEntry("SettingsPanel", "ValidHotkeyCombination");
            SettingsPanel.InvalidHotKeyCombination = GetEntry("SettingsPanel", "InvalidHotKeyCombination");

            // Location Panel
            SettingsPanel.lockDockAtCurrentPosition = GetEntry("SettingsPanel", "lockDockAtCurrentPosition");
            SettingsPanel.CentreAroundCursorWhenShown = GetEntry("SettingsPanel", "CentreAroundCursorWhenShown");

            // General Panel
            SettingsPanel.ConfigurationDescriptionLabel = GetEntry("SettingsPanel", "ConfigurationDescriptionLabel");
            SettingsPanel.KeepDockAboveAllOtherWindows = GetEntry("SettingsPanel", "KeepDockAboveAllOtherWindows");
            SettingsPanel.ClickSensitivity = GetEntry("SettingsPanel", "ClickSensitivity");
            SettingsPanel.Sensitive = GetEntry("SettingsPanel", "Sensitive");
            SettingsPanel.NotSensitive = GetEntry("SettingsPanel", "NotSensitive");
            SettingsPanel.EnablePortabilityMode = GetEntry("SettingsPanel", "EnablePortabilityMode");
            SettingsPanel.UseMemorySaver = GetEntry("SettingsPanel", "UseMemorySaver");
            SettingsPanel.useSameRotationValueAllLevels = GetEntry("SettingsPanel", "useSameRotationValueAllLevels");
            SettingsPanel.currentRotationValue = GetEntry("SettingsPanel", "currentRotationValue");

            SettingsPanel.Background = GetEntry("SettingsPanel", "Background");
            SettingsPanel.BackgroundImage = GetEntry("SettingsPanel", "BackgroundImage");
            SettingsPanel.NormalOpacity = GetEntry("SettingsPanel", "NormalOpacity");
            SettingsPanel.NormalSize = GetEntry("SettingsPanel", "NormalSize");

            // Dock Shape Panel
            SettingsPanel.ConstantNumberOfItemsPerCircle = GetEntry("SettingsPanel", "ConstantNumberOfItemsPerCircle");
            SettingsPanel.maximumNumberOfItemsPerCircle = GetEntry("SettingsPanel", "maximumNumberOfItemsPerCircle");
            SettingsPanel.NumberOfIconsPerCircle = GetEntry("SettingsPanel", "NumberOfIconsPerCircle");
            SettingsPanel.MinimumRadius = GetEntry("SettingsPanel", "MinimumRadius");
            SettingsPanel.SeparationBetweenCircles = GetEntry("SettingsPanel", "SeparationBetweenCircles");
            SettingsPanel.FormatWord = GetEntry("SettingsPanel", "FormatWord");
            SettingsPanel.CircleWord = GetEntry("SettingsPanel", "CircleWord");
            SettingsPanel.lineWord = GetEntry("SettingsPanel", "lineWord");
            SettingsPanel.spiralWord = GetEntry("SettingsPanel", "spiralWord");
            SettingsPanel.rateOfIncrease = GetEntry("SettingsPanel", "rateOfIncrease");
            SettingsPanel.defaultWord = GetEntry("SettingsPanel", "defaultWord");
            SettingsPanel.separationWord = GetEntry("SettingsPanel", "separationWord");

            SettingsPanel.CentreButton = GetEntry("SettingsPanel", "CentreButton");
            SettingsPanel.DefaultCentreButton = GetEntry("SettingsPanel", "DefaultCentreButton");
            SettingsPanel.OtherSettings = GetEntry("SettingsPanel", "OtherSettings");
            SettingsPanel.ShowStartMenuWhenClicked = GetEntry("SettingsPanel", "ShowStartMenuWhenClicked");
            SettingsPanel.StartMenuResidueWarning = GetEntry("SettingsPanel", "StartMenuResidueWarning");

            SettingsPanel.DockItems = GetEntry("SettingsPanel", "DockItems");
            SettingsPanel.RightClickMenu = GetEntry("SettingsPanel", "RightClickMenu");
            SettingsPanel.ShowWindowsFileFolderMenus = GetEntry("SettingsPanel", "ShowWindowsFileFolderMenus");
            SettingsPanel.HideDock = GetEntry("SettingsPanel", "HideDock");
            SettingsPanel.HideDockAfterFileExecution = GetEntry("SettingsPanel", "HideDockAfterFileExecution");

            // Labels Panel
            SettingsPanel.FontWord = GetEntry("SettingsPanel", "FontWord");
            SettingsPanel.ChangeFont = GetEntry("SettingsPanel", "ChangeFont");
            SettingsPanel.ChangeColor = GetEntry("SettingsPanel", "ChangeColor");
            SettingsPanel.ShadowWord = GetEntry("SettingsPanel", "ShadowWord");
            SettingsPanel.ShadowSize = GetEntry("SettingsPanel", "ShadowSize");
            SettingsPanel.ShadowDarknessFactor = GetEntry("SettingsPanel", "ShadowDarknessFactor");
            SettingsPanel.ShowLabels = GetEntry("SettingsPanel", "ShowLabels");
            SettingsPanel.ShowLabelsOnMouseoverOnly = GetEntry("SettingsPanel", "ShowLabelsOnMouseoverOnly");

            // Animation Panel
            SettingsPanel.AnimationWord = GetEntry("SettingsPanel", "AnimationWord");
            SettingsPanel.GeneralFrameRate = GetEntry("SettingsPanel", "GeneralFrameRate");
            SettingsPanel.RotationAnimationDuration = GetEntry("SettingsPanel", "RotationAnimationDuration");
            SettingsPanel.FadeInFadeOutDuration = GetEntry("SettingsPanel", "FadeInFadeOutDuration");
            SettingsPanel.UsePoofAnimationWhenDeleting = GetEntry("SettingsPanel", "UsePoofAnimationWhenDeleting");

            // About Panel
            SettingsPanel.AboutWord = GetEntry("SettingsPanel", "AboutWord");
            SettingsPanel.ThePremiereCircularDockForWindows = GetEntry("SettingsPanel", "ThePremiereCircularDockForWindows");
            SettingsPanel.WebsiteWord = GetEntry("SettingsPanel", "WebsiteWord");
            SettingsPanel.VersionInformation = GetEntry("SettingsPanel", "VersionInformation");
            SettingsPanel.OfficialHomepage = GetEntry("SettingsPanel", "OfficialHomepage");

            // Help Panel
            SettingsPanel.SupportWord = GetEntry("SettingsPanel", "SupportWord");
            SettingsPanel.OfficalWebForum = GetEntry("SettingsPanel", "OfficialWebForum");
            SettingsPanel.OfficialWebHelpFAQ = GetEntry("SettingsPanel", "OfficialWebHelpFAQ");

            // Language Panel
            SettingsPanel.LanguageFileLocation = GetEntry("SettingsPanel", "LanguageFileLocation");
            SettingsPanel.LanguageName = GetEntry("SettingsPanel", "LanguageName");
            SettingsPanel.LanguageFileAuthor = GetEntry("SettingsPanel", "LanguageFileAuthor");
            SettingsPanel.LanguageFileVersion = GetEntry("SettingsPanel", "LanguageFileVersion");
            SettingsPanel.LanguageFileOriginallyIntendedFor = GetEntry("SettingsPanel", "LanguageFileOriginallyIntendedFor");
            SettingsPanel.LanguageFileProgramRestartRequired = GetEntry("SettingsPanel", "LanguageFileProgramRestartRequired");
            SettingsPanel.LanguageFileValidCurrent = GetEntry("SettingsPanel", "LanguageFileValidCurrent");
            SettingsPanel.LanguageFileInvalidOutofDate = GetEntry("SettingsPanel", "LanguageFileInvalidOutofDate");

            // File -> Icon Associations Panel
            SettingsPanel.FileIconAssocationsDescription = GetEntry("SettingsPanel", "FileIconAssocationsDescription");
            SettingsPanel.FileNameIconAssociations = GetEntry("SettingsPanel", "FileNameIconAssociations");
            SettingsPanel.FolderNameIconAssociations = GetEntry("SettingsPanel", "FolderNameIconAssociations");
            SettingsPanel.FileExtensionIconAssociations = GetEntry("SettingsPanel", "FileExtensionIconAssociations");
            SettingsPanel.DockFolderIconAssociation = GetEntry("SettingsPanel", "DockFolderIconAssociation");

            SettingsPanel.fileName = GetEntry("SettingsPanel", "fileName");
            SettingsPanel.folderName = GetEntry("SettingsPanel", "folderName");
            SettingsPanel.extensionName = GetEntry("SettingsPanel", "extensionName");
            SettingsPanel.associatedDockIcon = GetEntry("SettingsPanel", "associatedDockIcon");
            SettingsPanel.browse = GetEntry("SettingsPanel", "browse");
            SettingsPanel.startsWith = GetEntry("SettingsPanel", "startsWith");
            SettingsPanel.endsWith = GetEntry("SettingsPanel", "endsWith");
            SettingsPanel.exactMatch = GetEntry("SettingsPanel", "exactMatch");
            SettingsPanel.contains = GetEntry("SettingsPanel", "contains");
            SettingsPanel.sample = GetEntry("SettingsPanel", "sample");

            // Toggling
            SettingsPanel.ToggleVisibilityHotKey = GetEntry("SettingsPanel", "ToggleVisibilityHotKey");
            SettingsPanel.ToggleVisibilityMouseButton = GetEntry("SettingsPanel", "ToggleVisibilityMouseButton");
            SettingsPanel.ShowDockWhenIMoveMyMouseTo = GetEntry("SettingsPanel", "ShowDockWhenIMoveMyMouseTo");
            SettingsPanel.ScreenLeftEdge = GetEntry("SettingsPanel", "ScreenLeftEdge");
            SettingsPanel.ScreenRightEdge = GetEntry("SettingsPanel", "ScreenRightEdge");
            SettingsPanel.ScreenTopEdge = GetEntry("SettingsPanel", "ScreenTopEdge");
            SettingsPanel.ScreenBottomEdge = GetEntry("SettingsPanel", "ScreenBottomEdge");
            SettingsPanel.EdgeWidthHeight = GetEntry("SettingsPanel", "EdgeWidthHeight");
            SettingsPanel.dwellTime = GetEntry("SettingsPanel", "dwellTime");
        }

        private void LoadErrorMessages()
        {
            ErrorMessages.MissingInvalid = GetEntry("ErrorMessages", "MissingInvalid");
            ErrorMessages.FilePathMissingInvalid = GetEntry("ErrorMessages", "FilePathMissingInvalid");
        }

        private void LoadDockItemProperties()
        {
            DockItemProperties.Arguments = GetEntry("DockItemProperties", "Arguments");
            DockItemProperties.ItemSettings = GetEntry("DockItemProperties", "ItemSettings");
            DockItemProperties.File = GetEntry("DockItemProperties", "File");
            DockItemProperties.Folder = GetEntry("DockItemProperties", "Folder");
            DockItemProperties.IconImage = GetEntry("DockItemProperties", "IconImage");
            DockItemProperties.Maximized = GetEntry("DockItemProperties", "Maximized");
            DockItemProperties.Minimized = GetEntry("DockItemProperties", "Minimized");
            DockItemProperties.Name = GetEntry("DockItemProperties", "Name");
            DockItemProperties.Normal = GetEntry("DockItemProperties", "Normal");
            DockItemProperties.Run = GetEntry("DockItemProperties", "Run");
            DockItemProperties.StartIn = GetEntry("DockItemProperties", "StartIn");
            DockItemProperties.Target = GetEntry("DockItemProperties", "Target");
            DockItemProperties.Close = GetEntry("DockItemProperties", "Close");
        }

        #endregion
    }
}
