﻿// Circle Dock 0.9.2 - Copyright 2008 Eric Wong
// Circle Dock is open source software licensed under GNU GENERAL PUBLIC LICENSE V3. See included Licence.txt file.
// http://circledock.wikidot.com
// VideoInPicture@gmail.com

// The program creates a circular-type dock with dock folders.
// Portions of the code are from various authors. See License.txt.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using System.IO;

using DockItemsInformation;
using AMS.Profile;
using BaseDockObjects;
using PerPixelAlphaForms;

using LanguageLoader;
using SettingsLoader;
using DockItemSettingsLoader;
using Pinvoke;

using Orbit.Utilities;
using Win32.Shell32;
using FileOps;

using GlobalMouseKeyboardHook;


namespace CircleDock
{
    /// <summary>
    /// MainForm is the central controller for Circle Dock. It is an invisible form that coordinates the creation, movement, disposal, etc.
    /// of each item in the dock. MainForm handles all the sizing of the dock items and determines where they should be placed.
    /// </summary>
    public partial class MainForm : PerPixelAlphaForms.InvisiblePerPixelAlphaForm 
    {
        // A string used to hold the version identifier of this build
        public readonly String CircleDockVersion = @"0.9.2 Alpha 8";
       
        #region Variables

        /// <summary>
        /// The object located in the centre of the dock. Used to move the dock around and show what folder is opened.
        /// </summary>
        public BaseDockObjects.CentreObject CentreObject;

        /// <summary>
        /// The background of the dock. Not selectable.
        /// </summary>
        public BaseDockObjects.BackgroundObject BackgroundObject;

        /// <summary>
        /// The collection of the dock items currently shown in the dock.
        /// </summary>
        public static ArrayList MainDockObjects;

        /// <summary>
        /// The language collection used for displayed words.
        /// </summary>
        public LanguageLoader.LanguageLoader Language;

        /// <summary>
        /// The general settings for the dock.
        /// </summary>
        public SettingsLoader.SettingsLoader DockSettings;

        /// <summary>
        /// Individual settings for each dock item.
        /// </summary>
        public static DockItemSettingsLoader.DockItemSettingsLoader DockItemSettings;

        /// <summary>
        /// The dock folder currently shown.
        /// </summary>
        public static String CurrentLevelShown;

        /// <summary>
        /// The stored position for each dock item.
        /// </summary>
        private static Point[] CalculatedPoints;

        /// <summary>
        /// The size of each of the dock items. Used to calculate position.
        /// </summary>
        private static Size[] NewDockItemSizes;

        /// <summary>
        /// Used to store how much the dock is rotated by.
        /// </summary>
        public double RotationValue;

        private SettingsPanel.DockSetup ConfigureSettings;

        public bool ToggleDockHotKeyStatus;

        public bool DockIsVisible;

        public bool DoShowDock;           // False if CircleDock is to be hidden when first run

        public bool AlwaysShowTopLevel; // True if top level icons are to be shown when Circle Dock is made visible

        public Cursor DeleteCursor;

        static UserActivityHook actHook;

        public int dwellTime = 0;

        public bool loaded = false;

        static Mutex MyMutex;   // For installers such as Inno Setup that can detect running programs from their Mutex

        #endregion

        #region Constructor and Initialization

        /// <summary>
        /// MainForm is the central controller for Circle Dock. It is an invisible form that coordinates the creation, movement, disposal, etc.
        /// of each item in the dock. MainForm handles all the sizing of the dock items and determines where they should be placed.
        /// 
        /// This constructor loads the settings and loads the language file. It initializes the background and centre objects as well as show
        /// the contents of folder "0-";
        /// </summary>
        public MainForm()
        {
            // Create a Mutex so that our Installer (Inno Seup) can check to see if CircleDock is
            // running and won't allow the user to install (new version) or uninstall whilst
            // CircleDock is in memory.
            MyMutex = new Mutex(true, "_CircleDock_");
            //
            // Load settings from files
            DockSettings = new SettingsLoader.SettingsLoader(Application.StartupPath + @"\System\Settings\Config.ini");
            LoadLanguage();
            DockItemSettings = new DockItemSettingsLoader.DockItemSettingsLoader(Application.StartupPath + @"\System\Settings\DockItemData.ini");

            InitializeComponent();

            DoubleBuffered = true;
            this.AnimationTimer.Tick += new System.EventHandler(this.AnimationTimer_Tick);
            AnimationTimer.Interval = DockSettings.General.AnimationInterval;

            DoShowDock = DockSettings.General.ShowDock;   // Is the Dock to be initially shown?
            AlwaysShowTopLevel = DockSettings.General.AlwaysShowTopLevel;

            this.MemoryManagementTimer.Tick += new System.EventHandler(this.MemoryManagementTimer_Tick);
            MemoryManagementTimer.Interval = 30*1000;
            ToggleMemorySaver(DockSettings.General.UseMemorySaver);
            CreateDeleteCursor();

            this.mouseTogglingDwellTimer.Tick += new EventHandler(mouseTogglingDwellTimer_Tick);
            mouseTogglingDwellTimer.Interval = 10;

            MainDockObjects = new ArrayList();

            InitializeBackgroundObject();
            InitializeCentreObject();

            //DrawCentreObject();
            DrawBackground();
            ShowLevel("0-");

            //BackgroundObject.Activate();

//            DockIsVisible = true;
            DockIsVisible = DoShowDock; //false;

            //Orbit.Hook.dll global mouse hook
            SetMouseKeys();

            //// C# mouse and keyboard global hooks
            actHook = new UserActivityHook(false, (int)MouseButtons.None, false);
            SetMouseScreenToggles();

            actHook.OnMouseActivity += new MouseEventHandler(MouseMoved);
            //actHook.KeyDown += new KeyEventHandler(MyKeyDown);
            //actHook.KeyPress += new KeyPressEventHandler(MyKeyPress);
            //actHook.KeyUp += new KeyEventHandler(MyKeyUp);
            //actHook.Start(false, (int)MouseButtons.Middle, false);
            //GC.KeepAlive(actHook);

            
            ToggleDockHotKeyStatus = false;
            RegisterHotKeyToggleDock();

            loaded = true;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            this.Visible = false;
        }

        /// <summary>
        /// We hide the MainForm when it is initially shown. Otherwise, it could be invisible but still be selectable.
        /// </summary>
        private void MainForm_Shown(object sender, EventArgs e)
        {
            this.Hide();
            DoShowDock = true;
        }

        public void ToggleMemorySaver(bool StartMemorySaver)
        {
            if (StartMemorySaver)
                MemoryManagementTimer.Start();
            else
                MemoryManagementTimer.Stop();
        }

        public void SetMouseKeys()
        {
            if (pauseMouseHotkeysToolStripMenuItem.Checked == false)
            {
                KeysConverter ConvertKeys = new KeysConverter();
                Keys Key1 = (Keys)ConvertKeys.ConvertFromString(DockSettings.Toggling.VisibilityMouseButton);

                if (Key1 != Keys.None)
                {
                    StartHook(this.Handle);
                    SetShortcutKey(Key1);
                }
                else
                {
                    StopHook(this.Handle);
                }
            }
            else
            {
                StopHook(this.Handle);
            }
        }

        public void PauseMouseKeys()
        {
            StopHook(this.Handle);
        }

        public void SetMouseScreenToggles()
        {
            if (DockSettings.Toggling.ScreenLeftEdgeShow || DockSettings.Toggling.ScreenTopEdgeShow || DockSettings.Toggling.ScreenRightEdgeShow
                || DockSettings.Toggling.ScreenBottomEdgeShow)
                actHook.Start(true, (int)MouseButtons.None, false);
            else
                actHook.Stop(true, true, false);
        }

        public void LoadLanguage()
        {
            if (Language == null)
            {
                if (DockSettings.Language.path.StartsWith(@".\"))
                {
                    Language = new LanguageLoader.LanguageLoader(Application.StartupPath + @DockSettings.Language.path.Substring(1, DockSettings.Language.path.Length - 1));
                }
                else
                {
                    Language = new LanguageLoader.LanguageLoader(@DockSettings.Language.path);
                }
            }
            else
            {
                Language.LoadLanguageData();
            }
        }

        /// <summary>
        /// Initializes the BackgroundObject and sets the image or draws the image.
        /// </summary>
        public void InitializeBackgroundObject()
        {
            Size tempSize;

            // Determine the canvas size to draw the background onto.
            if (DockSettings.Shape.Shape == "Ellipse")
            {
                tempSize = new Size((int)((float)DockSettings.EllipseParams.MinHeight * DockSettings.EllipseParams.AspectRatio), 
                    DockSettings.EllipseParams.MinHeight);
            }
            else //(DockSettings.Shape.Shape == "Circle")
            {
                tempSize = new Size(DockSettings.Background.DefaultWidth, DockSettings.Background.DefaultHeight);
            }

            BackgroundObject = new BaseDockObjects.BackgroundObject(this, Language, DockSettings, "Background", tempSize);
            //BackgroundObject.Owner = this;
            
            // MainForm handles all the location and sizing of the objects

            if (DockSettings.General.lockDockPosition)
                BackgroundObject.Location = DockSettings.General.lockedPosition;
            else
                BackgroundObject.Location = new Point((int)((double)SystemInformation.PrimaryMonitorSize.Width / 2.0 - (double)tempSize.Width / 2.0),
                    (int)((double)SystemInformation.PrimaryMonitorSize.Height / 2.0 - (double)tempSize.Height / 2.0));

            //Bitmap DrawBackground = new Bitmap(tempSize.Width, tempSize.Height);
            //using (Graphics gr = Graphics.FromImage(DrawBackground))
            //{
            //    gr.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            //    gr.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
            //    gr.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
            //    gr.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;

            //    Bitmap textureimage = new Bitmap(Application.StartupPath + @"\System\Background\Circle.png");
            //    Brush texturebrush = new TextureBrush(textureimage);
            //    Brush texturepenbrush = Brushes.LightGray;

            //    float penWidth = 5f;
            //    Pen texturepen = new Pen(texturepenbrush, penWidth);

            //    // Draw an ellipse with a pen
            //    gr.DrawEllipse(texturepen, penWidth, penWidth, (float)tempSize.Width - penWidth*2, (float)tempSize.Height - penWidth*2);

            //    // Draw a filled ellipse
            //    //gr.FillEllipse(texturebrush, 0, 0, tempSize.Width, tempSize.Height);

            //    // Set the image of the background object to the new background
            //    //BackgroundObject.SetBitmapManaged(ref DrawBackground);
            //}
           

        }

        public void DrawBackground()
        {
            Size tempSize = BackgroundObject.ObjectSize;
            if (DoShowDock == false)
            {
                BackgroundObject.DrawBitmapManaged(tempSize.Width, tempSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, 0);
            }
            else
            {
                BackgroundObject.DrawBitmapManaged(tempSize.Width, tempSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, DockSettings.Background.DefaultOpacity);
            }
            BackgroundObject.Show();
            BackgroundObject.BringToFront();
            BackgroundObject.Activate();
        }

        public void ResizeBackground(int NewWidth, int NewHeight)
        {
            Size CentreObjectSize = CentreObject.ObjectSize;

            BackgroundObject.DrawBitmapManaged(NewWidth, NewHeight, true, CentreObject.Location.X - (NewWidth / 2 - CentreObjectSize.Width / 2),
                CentreObject.Location.Y - (NewHeight / 2 - CentreObjectSize.Height / 2), false, 0, 0, 0, 0, false, 0);
        }

        public void ResizeCentreObject(int NewWidth, int NewHeight)
        {
            Size tempBackgroundObjectSize = BackgroundObject.ObjectSize;
            CentreObject.DrawBitmapManaged(NewWidth, NewHeight, true, BackgroundObject.Location.X + (int)((double)tempBackgroundObjectSize.Width / 2.0 - (double)NewWidth / 2.0),
                BackgroundObject.Location.Y + (int)((double)tempBackgroundObjectSize.Height / 2.0 - (double)NewHeight / 2.0), 
                false, 0, 0, 0, 0, false, 0); 
        }

        public void ResizeDockItems(int NewWidth, int NewHeight)
        {
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                tempDockItem.ObjectSize = new Size(NewWidth, NewHeight);
            }
            
            PositionCurrentLevel();
        }

        public void UpdateDockItemOpacity(int NewOpacity)
        {
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                Size tempDockItemSize = tempDockItem.ObjectSize;
                tempDockItem.DrawBitmapManaged(tempDockItemSize.Width, tempDockItemSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, NewOpacity);
            }
        }

        public void RefreshPortabilityMode()
        {
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                tempDockItem.InitializeFileOps();
                tempDockItem.SetBitmapRedraw();
            }
        }

        public void RedrawDockItemLabels()
        {
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                tempDockItem.MyLabel.RedrawBitmap();
            }
        }

        /// <summary>
        /// Initializes the CentreObject and sets its position relative to the background object. The CentreObject is 
        /// a child form of the BackgroundObject meaning that CentreObject will always be displayed on top of
        /// BackgroundObject.
        /// </summary>
        public void InitializeCentreObject()
        {
            Size tempSize = new Size(DockSettings.CentreImage.DefaultWidth, DockSettings.CentreImage.DefaultHeight);

            CentreObject = new BaseDockObjects.CentreObject(this, Language, DockSettings, "CentreImage", tempSize);
            CentreObject.Owner = BackgroundObject;

            // MainForm handles all the location and sizing of the objects
            Size tempBackgroundObjectSize = BackgroundObject.ObjectSize;
            CentreObject.Location = new Point(BackgroundObject.Location.X + (int)((double)tempBackgroundObjectSize.Width / 2.0 - (double)tempSize.Width / 2.0),
              BackgroundObject.Location.Y + (int)((double)tempBackgroundObjectSize.Height / 2.0 - (double)tempSize.Height / 2.0));

        }

        public void DrawCentreObject()
        {
            Size tempSize = CentreObject.ObjectSize;
            if (DoShowDock == false)
            {
                CentreObject.DrawBitmapManaged(tempSize.Width, tempSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, 0);
            }
            else
            {
                CentreObject.DrawBitmapManaged(tempSize.Width, tempSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, DockSettings.CentreImage.DefaultOpacity);
            }
            CentreObject.Show();
            CentreObject.BringToFront();
            CentreObject.Activate();
        }

        #endregion

        #region Dock Folder Manipulation

        /// <summary> 
        /// Closes all the dock folder specified by "Level". Examples of valid "Level" names are
        /// "0-", "0-1-", "0-3-4-", etc.
        /// </summary>
        /// <param name="Level">The name of the dock folder to be closed.</param>
        public void CloseLevel(String Level)
        {
            //HideDockItems();

            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                if (tempDockItem.DockItemSectionName.StartsWith(Level))
                {
                    tempDockItem.AnimationTimer.Stop();
                    tempDockItem.MyLabel.AnimationTimer.Stop();
                    tempDockItem.MyLabel.Close();
                    tempDockItem.Close();
                    MainDockObjects.RemoveAt(i);
                    
                    i--;
                }
            }
        }

        /// <summary> 
        /// Closes all dock items current shown.
        /// </summary>
        public void CloseLevel()
        {
            HideDockItems();

            foreach (BaseDockObjects.DockItemObject tempDockItem in MainDockObjects)
            {
                tempDockItem.AnimationTimer.Stop();
                tempDockItem.MyLabel.AnimationTimer.Stop();
                tempDockItem.MyLabel.Close();
                tempDockItem.Close();
            }
            MainDockObjects = new ArrayList();
        }

        /// <summary> 
        /// Shows the dock folder that is the parent of the current dock folder shown.
        /// </summary>
        public void ShowLevelUp()
        {
            if (showingLevel)
                return;

            if (CurrentLevelShown != null && CurrentLevelShown != "0-" && CurrentLevelShown.Length > 0)
            {
                int LastDash = -1;
                String NewLevel = CurrentLevelShown;
                for (int i = CurrentLevelShown.Length - 2; i >= 0; i--)
                {
                    if (CurrentLevelShown[i] == '-')
                    {
                        LastDash = i;
                        break;
                    }
                }

                if (LastDash > 0)
                {
                    NewLevel = CurrentLevelShown.Substring(0, LastDash + 1);
                }

                ShowLevel(NewLevel);
            }
        }

        public bool showingLevel = false;
        /// <summary> 
        /// Closes all the dock items shown and then shows the dock folder specified by "Level". Examples of valid "Level" names are
        /// "0-", "0-1-", "0-3-4-", etc.
        /// </summary>
        /// <param name="Level">The name of the dock folder from which to retrieve the dock items to be shown.</param>
        public void ShowLevel(String Level)
        {
            showingLevel = true;
            //CentreObject.TopMost = false;
            //BackgroundObject.TopMost = false;

            // Close the current folder shown and change the CenterObject image to the display image for "Level"
            CloseLevel(CurrentLevelShown);
            MainDockObjects = new ArrayList();
            
            CurrentLevelShown = Level;
            ChangeCentreImage(Level);

            String[] AllSectionNames = DockItemSettings.GetSectionNames();

            //for (int i = 0; i < AllSectionNames.Length; i++)
              //  Console.WriteLine("All Section Names + " + AllSectionNames.Length + "  ---- " + AllSectionNames[i]);

            loadRotationValue();

            // Go through each of the section names in DockItemSettings to see if it is an item of the folder "Level"
            for (int i = 0; AllSectionNames != null && i < AllSectionNames.Length; i++)
            {
                if (AllSectionNames[i].StartsWith(Level) && AllSectionNames[i].Substring(Level.Length, AllSectionNames[i].Length - Level.Length).Contains("-") == false)
                {
                    BaseDockObjects.DockItemObject NewDockItem = CreateDockItem(AllSectionNames[i], null);
                    if (DockSettings.General.ShowLevelRealTimeCreation) 
                        PositionCurrentLevel();

                    Application.DoEvents();
                }
            }

            //Console.WriteLine("maindockitems size: " + MainDockObjects.Count);
            BackgroundObject.Activate();
            BackgroundObject.BringToFront();
                //CentreObject.Activate();
                //CentreObject.BringToFront();

            if (!DockSettings.General.ShowLevelRealTimeCreation)
                PositionCurrentLevel();

            showingLevel = false;



            MemoryManagement.MemoryUtility.ClearUnusedMemory();

            
        }

        public void loadRotationValue()
        {
            if (DockSettings.General.useSameRotationValues)
            {
                try
                {
                    RotationValue = double.Parse(DockItemSettings.GetEntry(@"0", @"CircleRotation"));
                    if (double.IsNaN(RotationValue))
                    {
                        RotationValue = 0;
                        DockItemSettings.SetEntry(@"0", @"CircleRotation", RotationValue.ToString());
                    }
                }
                catch (Exception)
                {
                    RotationValue = 0;
                    DockItemSettings.SetEntry(@"0", @"CircleRotation", RotationValue.ToString());
                }
            }
            else
            {
                try
                {
                    RotationValue = double.Parse(DockItemSettings.GetEntry(CurrentLevelShown.Substring(0, CurrentLevelShown.Length - 1), @"CircleRotation"));
                    if (double.IsNaN(RotationValue))
                    {
                        RotationValue = 0;
                        DockItemSettings.SetEntry(CurrentLevelShown.Substring(0, CurrentLevelShown.Length - 1), @"CircleRotation", RotationValue.ToString());
                    }
                }
                catch (Exception)
                {
                    RotationValue = 0;
                    DockItemSettings.SetEntry(CurrentLevelShown.Substring(0, CurrentLevelShown.Length - 1), @"CircleRotation", RotationValue.ToString());
                }
            }
            TargetRotationValue = RotationValue;
        }


        public void saveRotationValue()
        {
            if (DockSettings.General.useSameRotationValues)
            {
                  DockItemSettings.SetEntry(@"0", @"CircleRotation", RotationValue.ToString());
            }
            else
            {
                  DockItemSettings.SetEntry(CurrentLevelShown.Substring(0, CurrentLevelShown.Length - 1), @"CircleRotation", RotationValue.ToString());
            }
        }

        public void resetDockLabels()
        {
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                tempDockItem.SetLabel("");
            }
        }

        public void reshowDockLabels()
        {
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                try
                {
                    tempDockItem.SetLabel(tempDockItem.FileOperations.Name);
                }
                catch (Exception)
                {
                    tempDockItem.SetLabel("");
                }

            }
        }

        /// <summary> 
        /// Calculates the current dock items positions and positions the current dock items. Function is dependant on MainDockItems.
        /// </summary>
        public void PositionCurrentLevel()
        {
            // Note: We must call CalculateDockItemPositions() after MainDockObjects.Sort() or we may get incorrect positioning
            NewDockItemSizes = new Size[MainDockObjects.Count];

            MainDockObjects.Sort(new DockItemOrderComparer(SortDirection.Ascending));
            // Retrive the display size of each of the matching dock items found.
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                tempDockItem.SetDockItemOrder(i);
                NewDockItemSizes[i] = tempDockItem.ObjectSize;
                //NewDockItemSizes[i] = new Size(DockSettings.DockItems.DefaultWidth, DockSettings.DockItems.DefaultHeight);
            }

            // Redraw the BackgroundObject to fit the new dock items.

            // Calculate the positions for each of the dock items.
            
            CalculatedPoints = CalculateDockItemPositions(NewDockItemSizes);

            // Create the new dock items
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                if (DoShowDock == false)
                {
                    tempDockItem.DrawBitmapManaged(NewDockItemSizes[i].Width, NewDockItemSizes[i].Height, true, CalculatedPoints[i].X, CalculatedPoints[i].Y, false, 0, 0, 0, 0, true, 0);
                }
                else
                {
                    tempDockItem.DrawBitmapManaged(NewDockItemSizes[i].Width, NewDockItemSizes[i].Height, true, CalculatedPoints[i].X, CalculatedPoints[i].Y, false, 0, 0, 0, 0, false, 0);
                    if ((DockSettings.Labels.ShowLabels) && (DockSettings.Labels.ShowLabelsOnMouseOverOnly == false))
                    {
                        try
                        {
                            tempDockItem.SetLabel(tempDockItem.FileOperations.Name);
                        }
                        catch (Exception)
                        {
                            tempDockItem.SetLabel("");
                        }
                    }
                }
            }
        }

        public void PositionCurrentLevelFast()
        {
            // Calculate the positions for each of the dock items.

            CalculatedPoints = CalculateDockItemPositions(NewDockItemSizes);

            // Create the new dock items
            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                tempDockItem.DrawBitmapManaged(NewDockItemSizes[i].Width, NewDockItemSizes[i].Height, true, CalculatedPoints[i].X, CalculatedPoints[i].Y, false, 0, 0, 0, 0, false, 0);
            }
        }

        /// <summary> 
        /// Changes the image displayed in CentreObject to the one specified by "Level". Examples of valid "Level" names are
        /// "0-", "0-1-", "0-3-4-", etc.
        /// </summary>
        /// <param name="Level">The name of the dock folder from which to retrieve the new display image.</param>
        public void ChangeCentreImage(String Level)
        {
            if (Level == null || Level.Length == 0)
                return;

            Bitmap NewBitmap;
            if (Level == "0-")
            {
                try
                {
                    if (DockSettings.CentreImage.Path.StartsWith(@".\"))
                    {
                        NewBitmap = new Bitmap(Application.StartupPath + @DockSettings.CentreImage.Path.Substring(1, DockSettings.CentreImage.Path.Length - 1));
                    }
                    else
                    {
                        NewBitmap = new Bitmap(@DockSettings.CentreImage.Path);
                    }
                }
                catch (Exception)
                {
                    NewBitmap = new Bitmap(ImageResources.CircleDockIconCentreImage);
                }
            }
            else
            {
                String ParsedLevel = Level.Substring(0, Level.Length - 1);
                FileOps.FileOps FileOperations;

                if (ParsedLevel != null && ParsedLevel != "")
                {
                    String ImagePath = @DockItemSettings.GetEntry(ParsedLevel, "ImagePath");

                    try
                    {
                        if (ImagePath != null && ImagePath.Length > 0)
                        {
                            if (ImagePath.StartsWith(@".\"))
                            {
                                NewBitmap = new Bitmap(Application.StartupPath + @ImagePath.Substring(1, ImagePath.Length - 1));
                            }
                            else
                            {
                                NewBitmap = new Bitmap(@ImagePath);
                            }
                        }
                        else
                        {
                            FileOperations = new FileOps.FileOps(IntPtr.Zero, @DockItemSettings.GetEntry(ParsedLevel, "Args"), Language, DockSettings);
                            NewBitmap = new Bitmap(FileOperations.RepresentativeImage);
                        }
                    }
                    catch (Exception)
                    {
                        NewBitmap = new Bitmap(ImageResources.CircleDockIconCentreImage);
                    }
                }
                else
                {
                    NewBitmap = new Bitmap(ImageResources.CircleDockIconCentreImage);
                }
            }
            CentreObject.SetBitmapManaged(ref NewBitmap);
            Size ObjectSize = CentreObject.ObjectSize;
            if (DoShowDock == false)
            {
                CentreObject.DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, 0);
            }
            else
            {
                CentreObject.DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, false, 0);
            }
        }

        #endregion

        #region Dock Item Manipulation

        /// <summary> 
        /// Sets the position for each of the dock items given by DesiredPositions.
        /// </summary>
        /// <param name="DesiredPositions">An array of points that specify where the dock items should be placed. The size of DesiredPositions needs to be the 
        /// same as MainDockObjects.</param>
        public void SetDockItemPositions(Point[] DesiredPositions)
        {
            if (DesiredPositions == null)
                return;

            for (int i = 0; i < DesiredPositions.Length && i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                tempDockItem.Location = new Point(DesiredPositions[i].X, DesiredPositions[i].Y);
            }  
        }

        /// <summary> 
        /// Creates a new dock item.
        /// </summary>
        /// <param name="SectionName">Unique identifier for the dock item in the DockItemSettings.</param>
        /// <param name="Path">The target link to execute. Specify null if none.</param>
        public BaseDockObjects.DockItemObject CreateDockItem(String SectionName, String Path)
        {
            Size tempSize = new Size(DockSettings.DockItems.DefaultWidth, DockSettings.DockItems.DefaultHeight);
            BaseDockObjects.DockItemObject NewDockItem = new BaseDockObjects.DockItemObject(this, Language, DockSettings, DockItemSettings, SectionName, tempSize, Path);

            NewDockItem.Owner = CentreObject;
            //NewDockItem.Activate();

            int DockOrder = 0;

            try
            {
                DockOrder = int.Parse(DockItemSettings.GetEntry(SectionName, "Order"));
            }
            catch (Exception)
            {
                DockOrder = 0;
            }

            MainDockObjects.Add(NewDockItem);
 
            return NewDockItem;
        }

        /// <summary> 
        /// Returns an array of positions at which the dock items should be drawn to fit on the background shape.
        /// </summary>
        /// <param name="DockItemSizes">Size of each of the dock items.</param>
        public Point[] CalculateDockItemPositions(Size[] DockItemSizes)
        {
            if (DockItemSizes == null)
                return new Point[0];

            Point[] CalculatedPoints = new Point[DockItemSizes.Length];
            if (DockSettings.Shape.Shape == "Ellipse")
            {
                // Need to write
                return null;
            }
            else //(DockSettings.Shape.Shape == "Circle")
            {
                if (DockSettings.CircleParams.Format == "Constant Number of Items Per Circle")
                {
                    Size tempCentreObjectSize = CentreObject.ObjectSize;
                    double RadianDivisons;

                    if (DockItemSizes.Length < DockSettings.CircleParams.ConstNumItemsPerCircle)
                        RadianDivisons = 2.0 * Math.PI / DockItemSizes.Length;
                    else
                    {
                        RadianDivisons = 2.0 * Math.PI / DockSettings.CircleParams.ConstNumItemsPerCircle;
                    }
                    int CentreObjectRadius;

                    if (tempCentreObjectSize.Height > tempCentreObjectSize.Width)
                        CentreObjectRadius = tempCentreObjectSize.Height / 2;
                    else
                        CentreObjectRadius = tempCentreObjectSize.Width / 2;

                    int IncreaseRadius = 0;
                    int tempCounter = 0;
                    for (int i = 0; i < DockItemSizes.Length; i++)
                    {
                        CalculatedPoints[i] = new Point(CentreObject.Location.X + CentreObjectRadius + (int)((double)(DockSettings.CircleParams.MinRadius + IncreaseRadius - DockItemSizes[i].Width / 2) * Math.Cos(i * RadianDivisons + RotationValue )) - DockItemSizes[i].Width / 2,
                            CentreObject.Location.Y + CentreObjectRadius + (int)((double)(DockSettings.CircleParams.MinRadius + IncreaseRadius - DockItemSizes[i].Height / 2) * Math.Sin(i * RadianDivisons + RotationValue )) - DockItemSizes[i].Height / 2);
                        tempCounter++;
                        if (tempCounter >= DockSettings.CircleParams.ConstNumItemsPerCircle)
                        {
                            tempCounter = 0;
                            IncreaseRadius = IncreaseRadius + DockSettings.CircleParams.CircleSeparation;
                        }
                    }

                    return CalculatedPoints;
                }
                else //(DockSettings.CircleParams.Format == "Maximum Number of Items Per Circle") 
                {
                    Size tempCentreObjectSize = CentreObject.ObjectSize;
                    double RadianDivisons;

                    int CentreObjectRadius;

                    if (tempCentreObjectSize.Height > tempCentreObjectSize.Width)
                        CentreObjectRadius = tempCentreObjectSize.Height / 2;
                    else
                        CentreObjectRadius = tempCentreObjectSize.Width / 2;

                    int IncreaseRadius = 0;

                    int maxNumItems = (int)Math.Floor(2.0 * Math.PI * (double)(DockSettings.CircleParams.MinRadius + IncreaseRadius - DockSettings.DockItems.DefaultWidth / 2) /
                        Math.Sqrt(Math.Pow((double)DockSettings.DockItems.DefaultWidth, 2.0) + Math.Pow((double)DockSettings.DockItems.DefaultHeight, 2.0)));

                    if (maxNumItems > MainDockObjects.Count)
                        maxNumItems = MainDockObjects.Count;
                    if (maxNumItems <= 0)
                        maxNumItems = 1;
                    RadianDivisons = 2.0 * Math.PI / maxNumItems;

                    int tempCounter = 0;
                    for (int i = 0; i < DockItemSizes.Length; i++)
                    {
                        CalculatedPoints[i] = new Point(CentreObject.Location.X + CentreObjectRadius + (int)((double)(DockSettings.CircleParams.MinRadius + IncreaseRadius - DockItemSizes[i].Width / 2) * Math.Cos(i * RadianDivisons + RotationValue)) - DockItemSizes[i].Width / 2,
                            CentreObject.Location.Y + CentreObjectRadius + (int)((double)(DockSettings.CircleParams.MinRadius + IncreaseRadius - DockItemSizes[i].Height / 2) * Math.Sin(i * RadianDivisons + RotationValue)) - DockItemSizes[i].Height / 2);
                        tempCounter++;
                        if (tempCounter >= maxNumItems)
                        {
                            tempCounter = 0;
                            IncreaseRadius = IncreaseRadius + DockSettings.CircleParams.CircleSeparation;

                            maxNumItems = (int)Math.Floor(2.0 * Math.PI * (double)(DockSettings.CircleParams.MinRadius + IncreaseRadius - DockSettings.DockItems.DefaultWidth / 2) /
                                                    Math.Sqrt(Math.Pow((double)DockSettings.DockItems.DefaultWidth, 2.0) + Math.Pow((double)DockSettings.DockItems.DefaultHeight, 2.0)));
                            if (maxNumItems <= 0)
                                maxNumItems = 1;
                            RadianDivisons = 2.0 * Math.PI / maxNumItems;
                        }
                    }

                    return CalculatedPoints;
                }
            }
        }

        /// <summary> 
        /// Completely removes a dock item from the display and from DockItemSettings.
        /// </summary>
        /// <param name="ItemToRemove">The dock item that is currently open and you want to remove.</param>
        public void RemoveDockItem(BaseDockObjects.DockItemObject ItemToRemove)
        {
            String ItemToRemoveSectionHeader = @ItemToRemove.DockItemSectionName + @"-";
            String TmpItemToRemove = @ItemToRemove.DockItemSectionName;
            CloseLevel(@ItemToRemoveSectionHeader);
            MainDockObjects.Remove(ItemToRemove);
            ItemToRemove.Close();

            String[] AllSectionNames = DockItemSettings.GetSectionNames();

            for (int i = 0; i < AllSectionNames.Length; i++)
            {
                if (AllSectionNames[i].StartsWith(ItemToRemoveSectionHeader) && 
                    AllSectionNames[i].Substring(ItemToRemoveSectionHeader.Length, AllSectionNames[i].Length - ItemToRemoveSectionHeader.Length).Contains("-") == false)
                {
                    DockItemSettings.RemoveSection(AllSectionNames[i]);
                }
            }

            DockItemSettings.RemoveSection(ItemToRemoveSectionHeader.Substring(0, ItemToRemoveSectionHeader.Length - 1));

            PositionCurrentLevel();

        }

        /// <summary> 
        /// Creates a new dock item from the given data. Function creates a new section in DockItemSettings with a unique
        /// identifier. The new dock item is a child of the folder currently opened. Updated to allow for the easy creation
        /// of Folder Stacks.
        /// </summary>
        /// <param name="Action">Action to be taken by object when invoked.</param>
        /// <param name="Args">Arguments for the action. Typically the link to the file to open or execute.</param>
        /// <param name="Description">Description for the dock item.</param>
        /// <param name="Name">Display name for the dock item.</param>
        public void AddDockItem(String Action, String Args, String Description, String ImagePath, String Name)
        {
            String originalAction = Action;

            if ((Action == @"[blanklink]") || (Action == @"[stack]"))
                Action = @"[link]";

            String[] AllSectionHeaders = DockItemSettings.GetSectionNames();
            int UnusedNumber = 0;
            ArrayList UsedNumbers = new ArrayList();

            // Generate a unique identifier.
            foreach (String tempHeader in AllSectionHeaders)
            {
                if (tempHeader.StartsWith(CurrentLevelShown))
                {
                    String tempHeaderSubString = tempHeader.Substring(CurrentLevelShown.Length, tempHeader.Length - CurrentLevelShown.Length);
                    int NextDashLocation = tempHeaderSubString.IndexOf('-');
                    if (NextDashLocation < 0)
                        NextDashLocation = tempHeaderSubString.Length;

                    int CurrentNumber = int.Parse(tempHeaderSubString.Substring(0, NextDashLocation));
                    UsedNumbers.Add(CurrentNumber);
                }
            }

            UsedNumbers.Sort();
            for (int i = 0; i < UsedNumbers.Count; i++)
            {
                if (UnusedNumber == (int)(UsedNumbers[i]))
                {
                    UnusedNumber++;
                    i = 0;
                }
            }

            String NewSectionHeader = CurrentLevelShown + UnusedNumber.ToString();
            String ArgsParams = "";
            String RunAs = @"Normal";

            if (Action == @"[link]" && Args.ToUpper().EndsWith(@".LNK") && System.IO.File.Exists(Args))
            {
                try
                {
                    Win32.Shell32.SHFileInfo FileInfo = new Win32.Shell32.SHFileInfo();
                    Win32.Shell32.Shell32API.SHGetFileInfo(Args, 0, ref FileInfo, (uint)System.Runtime.InteropServices.Marshal.SizeOf(FileInfo),
                        Win32.Shell32.ShellGetFileInfoFlags.DisplayName | Win32.Shell32.ShellGetFileInfoFlags.FileTypeName);

                    string pathOnly = System.IO.Path.GetDirectoryName(Args);
                    string filenameOnly = System.IO.Path.GetFileName(Args);

                    Shell32.Shell shell = new Shell32.ShellClass();
                    Shell32.Folder folder = shell.NameSpace(pathOnly);
                    Shell32.FolderItem folderItem = folder.ParseName(filenameOnly);
                    if (folderItem != null)
                    {
                        Shell32.ShellLinkObject link = (Shell32.ShellLinkObject)folderItem.GetLink;

                        String advertisedShortcutTarget = IconsAndShortcuts.MsiShortcutParser.ParseShortcut(Args);

                        if (advertisedShortcutTarget != null)
                            Args = advertisedShortcutTarget;
                        else if (link.Path.Length > 0)
                            Args = link.Path;

                        ArgsParams = link.Arguments;

                        if (link.ShowCommand == 3)
                            RunAs = @"Maximized";
                        else if (link.ShowCommand == 7)
                            RunAs = @"Minimized";
                        else
                            RunAs = @"Normal";
                    }

                    if (Name == @"")
                    {
                        // set the item name to the file name
                        Name = FileInfo.szDisplayName;
                    }

                    if (Description == @"")
                    {
                        // set the item description to the file type
                        Description = FileInfo.szTypeName;
                    }
                }
                catch (Exception)
                { }
            }

            if (Action == @"[link]" && Args.StartsWith(Application.StartupPath + @"\"))
            {
                Args = @"." + Args.Substring(Application.StartupPath.Length);
                //Console.WriteLine(Args);
            }

            if (originalAction == @"[stack]")
                ArgsParams = "/notaskbar /right";

            // Initializes the dock item settings.
            DockItemSettings.SetEntry(NewSectionHeader, @"Action", Action);
            DockItemSettings.SetEntry(NewSectionHeader, @"Args", Args);
            DockItemSettings.SetEntry(NewSectionHeader, @"Description", Description);
            DockItemSettings.SetEntry(NewSectionHeader, @"ImagePath", ImagePath);
            DockItemSettings.SetEntry(NewSectionHeader, @"Name", Name);
            DockItemSettings.SetEntry(NewSectionHeader, @"Order", (MainDockObjects.Count).ToString());
            DockItemSettings.SetEntry(NewSectionHeader, @"CircleRotation", @"0");
            DockItemSettings.SetEntry(NewSectionHeader, @"EllipseRotation", @"0");
            DockItemSettings.SetEntry(NewSectionHeader, @"SpiralRotation", @"0");
            DockItemSettings.SetEntry(NewSectionHeader, @"ArgsParams", ArgsParams);
            DockItemSettings.SetEntry(NewSectionHeader, @"RunAs", RunAs);

            // Display the new dock item to the screen.
            BaseDockObjects.DockItemObject NewDockItem = CreateDockItem(NewSectionHeader, null);

            if ((originalAction == @"[blanklink]") || (originalAction == @"[stack]"))
                NewDockItem.itemSettingsToolStripMenuItem_Click(this, null);
        }

        public bool AtRootLevel()
        {
            if (CurrentLevelShown == "0-")
                return true;
            else
                return false;
        }

        #endregion

        #region Movement Handlers

        public void ChildMoved(BaseDockObjects.DockItemObject ChildObject)
        {
            Size CentreObjectSize = CentreObject.ObjectSize;

            if (Cursor.Position.X >= CentreObject.Location.X && Cursor.Position.Y >= CentreObject.Location.Y
                && Cursor.Position.X <= CentreObject.Location.X + CentreObjectSize.Width && Cursor.Position.Y <= CentreObject.Location.Y + CentreObjectSize.Height)
            {
                Cursor.Current = Cursors.Default;

                if (DockSettings.PoofAnimation.UsePoof)
                {
                    ChildObject.MyLabel.Hide();
                    Bitmap poofBitmapWhole;
                    try
                    {
                        if (DockSettings.PoofAnimation.PoofAnimationPath.StartsWith(@".\"))
                        {
                            poofBitmapWhole = new Bitmap(Application.StartupPath + @DockSettings.PoofAnimation.PoofAnimationPath.Substring(1, DockSettings.PoofAnimation.PoofAnimationPath.Length - 1));
                        }
                        else
                        {
                            poofBitmapWhole = new Bitmap(@DockSettings.PoofAnimation.PoofAnimationPath);
                        }
                    }
                    catch (Exception)
                    {
                        poofBitmapWhole = new Bitmap(ImageResources.Poof);
                    }

                    Size poofSize = new Size(poofBitmapWhole.Width / DockSettings.PoofAnimation.NumFrames, poofBitmapWhole.Height);

                    Bitmap poofBitmapPartial = new Bitmap(poofSize.Width, poofSize.Height);

                    Graphics gr = Graphics.FromImage(poofBitmapPartial);
                    gr.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                    gr.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                    gr.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
                    gr.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;

                    for (int i = 0; i < DockSettings.PoofAnimation.NumFrames; i++)
                    {
                        gr.Clear(Color.Transparent);
                        gr.DrawImage(poofBitmapWhole, new Rectangle(0, 0, poofBitmapPartial.Width, poofBitmapPartial.Height),
                            new Rectangle(i * poofSize.Width, 0, i * poofSize.Width + poofSize.Width, poofBitmapWhole.Height), GraphicsUnit.Pixel);
                        Bitmap poofBitmapPartialTemp = new Bitmap(poofBitmapPartial);
                        ChildObject.SetBitmapManaged(ref poofBitmapPartialTemp);
                        ChildObject.DrawBitmapManaged(poofSize.Width, poofSize.Height, false, 0, 0, false, 0, 0, 0, 0, false, 0);

                        Thread.Sleep(DockSettings.PoofAnimation.DelayBetweenFrames);
                    }
                }

                RemoveDockItem(ChildObject);
            }
            else
            {
                for (int i = 0; i < MainDockObjects.Count; i++)
                {
                    BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                    Size DockItemSize = tempDockItem.ObjectSize;

                    if (Cursor.Position.X >= tempDockItem.Location.X && Cursor.Position.Y >= tempDockItem.Location.Y
                        && Cursor.Position.X <= tempDockItem.Location.X + DockItemSize.Width && Cursor.Position.Y <= tempDockItem.Location.Y + DockItemSize.Height
                        && ChildObject != tempDockItem)
                    {
                        ChildObject.SetDockItemOrder(tempDockItem.dockOrderNum);
                        break;
                    }
                }

                PositionCurrentLevel();
            }
        }

        public void CreateDeleteCursor()
        {
            Bitmap DeleteCursorBitmap = ImageOperations.BitmapOperations.ScaleBySize(CircleDock.ImageResources.Garbage_Can, 32, 32);
            DeleteCursor = new Cursor(DeleteCursorBitmap.GetHicon());
        }
        
        public void ChildMoving(BaseDockObjects.DockItemObject ChildObject)
        {
            Size CentreObjectSize = CentreObject.ObjectSize;
            if (Cursor.Position.X >= CentreObject.Location.X && Cursor.Position.Y >= CentreObject.Location.Y
                && Cursor.Position.X <= CentreObject.Location.X + CentreObjectSize.Width && Cursor.Position.Y <= CentreObject.Location.Y + CentreObjectSize.Height)
            {
                ChildObject.Cursor = DeleteCursor;
            }
            else
            {
                ChildObject.Cursor = Cursors.Default;
            }
        }

        public void ChildMouseEnter(BaseDockObjects.DockItemObject ChildObject)
        {
        }

        public void ChildMouseLeave(BaseDockObjects.DockItemObject ChildObject)
        {
        }

        public void ChildMoved(BaseDockObjects.BackgroundObject ChildObject)
        {
            Size tempBackgroundObjectSize = BackgroundObject.ObjectSize;
            Size tempCentreObjectSize = CentreObject.ObjectSize;
            CentreObject.Location = new Point(BackgroundObject.Location.X + (int)((double)tempBackgroundObjectSize.Width / 2.0 - (double)tempCentreObjectSize.Width / 2.0),
              BackgroundObject.Location.Y + (int)((double)tempBackgroundObjectSize.Height / 2.0 - (double)tempCentreObjectSize.Height / 2.0));

            CalculatedPoints = CalculateDockItemPositions(NewDockItemSizes);
            SetDockItemPositions(CalculatedPoints);
        }

        /// <summary> 
        /// Handles the event of when the CentreObject is moved by the user. Used to move the dock around.
        /// </summary>
        /// <param name="ChildObject">The CentreObject moved.</param>
        public void ChildMoved(BaseDockObjects.CentreObject ChildObject)
        {
            Size BackgroundSize = BackgroundObject.ObjectSize;
            Size CentreObjectSize = ChildObject.ObjectSize;

            BackgroundObject.Location = new Point(ChildObject.Location.X - (BackgroundSize.Width / 2 - CentreObjectSize.Width / 2),
                ChildObject.Location.Y - (BackgroundSize.Height / 2 - CentreObjectSize.Height / 2));

            CalculatedPoints = CalculateDockItemPositions(NewDockItemSizes);
            SetDockItemPositions(CalculatedPoints);
        }



        public bool RotateClockwise;
        public double TargetRotationValue;
        public int RotationDuration;
        /// <summary> 
        /// Rotates the dock.
        /// </summary>
        /// <param name="increment">Positive or negative.</param>
        public void RotateDock(double increment)
        {
            if (DockSettings.General.EnableDockRotation == false)
                return;

            if (DockSettings.Shape.Shape == "Ellipse")
            {
                // Need to write
            }
            else //(DockSettings.Shape.Shape == "Circle")
            {
                if (increment < 0)
                {
                    RotateClockwise = true;
                    if (TargetRotationValue > double.MinValue + 10000)
                        TargetRotationValue = TargetRotationValue - (2 * Math.PI / DockSettings.General.KeyPressesPerRevolution);
                }
                else
                {
                    RotateClockwise = false;
                    if (TargetRotationValue < double.MaxValue - 10000)
                        TargetRotationValue = TargetRotationValue + (2 * Math.PI / DockSettings.General.KeyPressesPerRevolution);
                }

                AnimationTimer.Start();

                // Use code below for instantaneous dock rotation
                //if (increment < 0)
                //{
                //    RotationValue -= (2 * Math.PI / DockSettings.CircleParams.KeyPressesPerRevolution);
                //}
                //else
                //{
                //    RotationValue += (2 * Math.PI / DockSettings.CircleParams.KeyPressesPerRevolution);
                //}

                //// Keep RotationValue between -2*Pi and 2*Pi
                //int NumRevs = 0;
                //if (RotationValue > 2 * Math.PI)
                //{
                //    NumRevs = (int)(RotationValue / (2 * Math.PI));
                //}

                //if (RotationValue < -2 * Math.PI)
                //{
                //    NumRevs = (int)(RotationValue / (2 * Math.PI));
                //}

                //RotationValue = RotationValue - NumRevs * (2 * Math.PI);

                //CalculatedPoints = CalculateDockItemPositions(NewDockItemSizes);

                //for (int i = 0; i < MainDockObjects.Count; i++)
                //{
                //    BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                //    tempDockItem.Location = new Point(CalculatedPoints[i].X, CalculatedPoints[i].Y);
                //}

                saveRotationValue();
            }
            
        }

        private void AnimationTimer_Tick(object sender, EventArgs e)
        {
            if (DockSettings.General.EnableDockRotation == false)
            {
                AnimationTimer.Stop();
                return;
            }

            if (DockSettings.Shape.Shape == "Ellipse")
            {
                // Need to write
                AnimationTimer.Stop();
            }
            else //(DockSettings.Shape.Shape == "Circle")
            {
                if (RotateClockwise)
                {
                    RotationValue -= (2.0 * Math.PI / ((double)DockSettings.GeneralAnimation.RotationAnimationDuration / (double)DockSettings.General.AnimationInterval));
                    if (RotationValue < TargetRotationValue)
                        RotationValue = TargetRotationValue;
                }
                else
                {
                    RotationValue += (2.0 * Math.PI / ((double)DockSettings.GeneralAnimation.RotationAnimationDuration / (double)DockSettings.General.AnimationInterval));
                    if (RotationValue > TargetRotationValue)
                        RotationValue = TargetRotationValue;
                }

                
                // Keep RotationValue between -2*Pi and 2*Pi
                //int NumRevs = 0;
                //if (RotationValue >= 2 * Math.PI)
                //{
                //    NumRevs = (int)(RotationValue / (2 * Math.PI));
                //}

                //if (RotationValue <= -2 * Math.PI)
                //{
                //    NumRevs = (int)(RotationValue / (2 * Math.PI));
                //}

                //RotationValue = RotationValue - NumRevs * (2 * Math.PI);

                CalculatedPoints = CalculateDockItemPositions(NewDockItemSizes);

                for (int i = 0; i < MainDockObjects.Count; i++)
                {
                    BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                    tempDockItem.Location = new Point(CalculatedPoints[i].X, CalculatedPoints[i].Y);
                }

                if ((RotateClockwise && RotationValue <= TargetRotationValue) || (RotateClockwise == false && RotationValue >= TargetRotationValue))
                {
                    AnimationTimer.Stop();

                    int NumRevs = 0;
                    if (RotationValue >= 2 * Math.PI)
                    {
                        NumRevs = (int)(RotationValue / (2 * Math.PI));
                    }

                    if (RotationValue <= -2 * Math.PI)
                    {
                        NumRevs = (int)(RotationValue / (2 * Math.PI));
                    }

                    RotationValue = RotationValue - NumRevs * (2 * Math.PI);

                    TargetRotationValue = RotationValue;
                }
            }
        }

        public void UpdateAnimationIntervals(int NewInterval)
        {
            this.AnimationTimer.Interval = NewInterval;
            BackgroundObject.UpdateAnimationTimerInterval(NewInterval);
            CentreObject.UpdateAnimationTimerInterval(NewInterval);

            for (int i = 0; i < MainDockObjects.Count; i++)
            {
                BaseDockObjects.DockItemObject tempDockItem = (BaseDockObjects.DockItemObject)MainDockObjects[i];
                tempDockItem.UpdateAnimationTimerInterval(NewInterval);
            }
        }

        private void MemoryManagementTimer_Tick(object sender, EventArgs e)
        {
            MemoryManagement.MemoryUtility.ClearUnusedMemory();
        }

        #endregion

        #region Tray Icon Context Menu

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void showToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowAll();
        }

        private void hideToolStripMenuItem_Click(object sender, EventArgs e)
        {
            HideAll();
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowSettingsPanel();
        }

        private void iconReplacementModeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DockSettings.General.IconReplacementMode = iconReplacementModeToolStripMenuItem.Checked;
            DockSettings.SetEntry("General", "IconReplacementMode", iconReplacementModeToolStripMenuItem.Checked.ToString());

            if (DockSettings.General.IconReplacementMode)
            {
                FileOps.FileOps FileOperations = new FileOps.FileOps(IntPtr.Zero, Language, DockSettings);
                FileOperations.Open(Application.StartupPath + @"\System\Icons", @"", System.Diagnostics.ProcessWindowStyle.Normal, IntPtr.Zero);
            }
           
        }

        private void pauseMouseHotkeysToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetMouseKeys();
        }

        private void MainFormContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            iconReplacementModeToolStripMenuItem.Checked = DockSettings.General.IconReplacementMode;
        }   

        private void MainFormTrayIcon_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
                ToggleVisiblity();
        }

        #endregion

        #region Miscellaneous

        /// <summary> 
        /// The child objects can request program actions to be taken.
        /// This is currently not fully utilized.
        /// 
        /// The intention to require the dock items to tell the MainForm to quit is so that 
        /// things can be finalized before closing.
        /// </summary>
        /// <param name="ActionCommand">Command to execute.</param>
        public void ChildRequestAction(String ActionCommand)
        {
            switch (ActionCommand)
            {
                case "QUIT":
                    this.Close();
                    break;
                case "TAKE_FOCUS":
                    CentreObject.Focus();
                    break;
            }
        }

        /// <summary>
        /// Finalizes things before closing.
        /// </summary>
        private void MainForm_FormClosing(object sender, EventArgs e)
        {
            StopHook(this.Handle);
            actHook.Stop(true, true, false);
            UnRegisterHotKeyToggleDock();
            MyMutex.ReleaseMutex();         // For installers checking the Mutex
        }

        public void ShowSettingsPanel()
        {
            // Don't allow the Dock to hide itself!
            BackgroundObject.DockCanHide = false;
            if (ConfigureSettings == null || ConfigureSettings.IsDisposed)
            {
                ConfigureSettings = new SettingsPanel.DockSetup(this, Language, DockSettings, DockItemSettings);
                ConfigureSettings.Show();
                ConfigureSettings.BringToFront();
            }
            else
            {
                ConfigureSettings.Show();
                ConfigureSettings.BringToFront();
            }
        }

        public bool LanguageFileIsValid()
        {
            String[] CompatibleCircleDockVersions = { CircleDockVersion };
            String NormalizedLanguageCircleDockVersion = Language.LanguageFile.CircleDockVersion.Trim().ToUpper();
            foreach (String CompareVersion in CompatibleCircleDockVersions)
            {
                if (CompareVersion.Trim().ToUpper() == NormalizedLanguageCircleDockVersion)
                    return true;    
            }

            return false;
        }

        #endregion

        #region Visibility

        /// <summary>
        /// Make all the items in the dock visible.
        /// </summary>
        public void ShowAll()
        {
            //Console.WriteLine("Show");
            DockIsVisible = true;
            if (DockSettings.General.zLevel == "Always On Bottom")
            {
                //foreach (BaseDockObjects.DockItemObject tempDockItem in MainDockObjects)
                //{
                //    tempDockItem.Activate();
                //    tempDockItem.MyLabel.Activate();
                //}

                BackgroundObject.Activate();
                BackgroundObject.BringToFront();
                //CentreObject.Activate();
                //CentreObject.BringToFront();
            }
            else
            {
                // We must activate each item when shown or Circle Dock may not get focus when shown

                BackgroundObject.AnimateOpacity(DockSettings.Background.DefaultOpacity, DockSettings.GeneralAnimation.FadeInFadeOutDuration);

                CentreObject.AnimateOpacity(DockSettings.CentreImage.DefaultOpacity, DockSettings.GeneralAnimation.FadeInFadeOutDuration);


                foreach (BaseDockObjects.DockItemObject tempDockItem in MainDockObjects)
                {
                    tempDockItem.AnimateOpacity(DockSettings.DockItems.DefaultOpacity, DockSettings.GeneralAnimation.FadeInFadeOutDuration);
                    tempDockItem.MyLabel.AnimateOpacity(DockSettings.DockItems.DefaultOpacity, DockSettings.GeneralAnimation.FadeInFadeOutDuration);
                    //tempDockItem.Activate();
                    //tempDockItem.MyLabel.Activate();
                }

                BackgroundObject.Activate();
                BackgroundObject.BringToFront();
                CentreObject.Activate();
                CentreObject.BringToFront();
                CentreObject.StartPosition = FormStartPosition.CenterScreen;
            }
        }

        /// <summary>
        /// Hide all the items in the dock, including the centre button and background.
        /// </summary>
        public void HideAll()
        {
            //Console.WriteLine("hide");
            DockIsVisible = false;
            if (DockSettings.General.zLevel == "Always On Bottom")
            {
                foreach (BaseDockObjects.DockItemObject tempDockItem in MainDockObjects)
                {
                    Pinvoke.Win32.SetWindowPos(tempDockItem.Handle, (IntPtr)Pinvoke.Win32.HWND_BOTTOM, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                        | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
                    Pinvoke.Win32.SetWindowPos(tempDockItem.MyLabel.Handle, (IntPtr)Pinvoke.Win32.HWND_BOTTOM, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                        | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
                    //tempDockItem.SetZLevel();
                    //tempDockItem.MyLabel.SetZLevel();
                }

                Pinvoke.Win32.SetWindowPos(CentreObject.Handle, (IntPtr)Pinvoke.Win32.HWND_BOTTOM, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                        | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);

                Pinvoke.Win32.SetWindowPos(BackgroundObject.Handle, (IntPtr)Pinvoke.Win32.HWND_BOTTOM, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                        | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);

                //BackgroundObject.SetZLevel();
                //CentreObject.SetZLevel();
            }
            else
            {
                foreach (BaseDockObjects.DockItemObject tempDockItem in MainDockObjects)
                {
                    tempDockItem.AnimateOpacity(0, DockSettings.GeneralAnimation.FadeInFadeOutDuration);
                    tempDockItem.MyLabel.AnimateOpacity(0, DockSettings.GeneralAnimation.FadeInFadeOutDuration);
                }

                BackgroundObject.AnimateOpacity(0, DockSettings.GeneralAnimation.FadeInFadeOutDuration);
                CentreObject.AnimateOpacity(0, DockSettings.GeneralAnimation.FadeInFadeOutDuration);
            }
        }

        /// <summary>
        /// Hide all the items in the dock.
        /// </summary>
        public void HideDockItems()
        {
            foreach (BaseDockObjects.DockItemObject tempDockItem in MainDockObjects)
            {
                tempDockItem.AnimateOpacity(0, DockSettings.GeneralAnimation.FadeInFadeOutDuration);
                tempDockItem.MyLabel.AnimateOpacity(0, DockSettings.GeneralAnimation.FadeInFadeOutDuration);
            }
        }

        /// <summary>
        /// Toggles the visiblity of the dock.
        /// </summary>
        public void ToggleVisiblity()
        {
            if (DockIsVisible)
                HideAll();
            else
            {
                // If the user wants the top level "menu" items to be displayed
                // each time the Dock is made visible...
                if (DockSettings.General.AlwaysShowTopLevel == true)
                    ShowLevel("0-");
                // Otherwise the current "menu" level menu level is displayed
                Size BackgroundSize = BackgroundObject.ObjectSize;
                Size CentreObjectSize = CentreObject.ObjectSize;
                // If the position must be at the current cursor position ...
                if (DockSettings.General.lockDockPosition == false && DockSettings.General.CentreAroundCursor)
                {
//                    Size BackgroundSize = BackgroundObject.ObjectSize;
//                    Size CentreObjectSize = CentreObject.ObjectSize;

                    CentreObject.Location = new Point(Cursor.Position.X - CentreObjectSize.Width / 2, Cursor.Position.Y - CentreObjectSize.Height / 2);

                    CalculatedPoints = CalculateDockItemPositions(NewDockItemSizes);
                    SetDockItemPositions(CalculatedPoints);

                    BackgroundObject.Location = new Point(CentreObject.Location.X - (BackgroundSize.Width / 2 - CentreObjectSize.Width / 2),
                        CentreObject.Location.Y - (BackgroundSize.Height / 2 - CentreObjectSize.Height / 2));
                }
                // If the position is neither locked or to be at the current cursor position, then ensure entire
                // Dock is completely on-screen.
                if (DockSettings.General.lockDockPosition == false && DockSettings.General.CentreAroundCursor == false)
                {
                    Point BestCOPosition = new Point(Cursor.Position.X - CentreObjectSize.Width / 2, Cursor.Position.Y - CentreObjectSize.Height / 2);

                    if (BestCOPosition.Y - (BackgroundSize.Height / 2) < SystemInformation.VirtualScreen.Top)
                        BestCOPosition.Y = (BackgroundSize.Height / 2) - (CentreObjectSize.Height / 2);

                    if (BestCOPosition.Y + (BackgroundSize.Height / 2) > SystemInformation.VirtualScreen.Bottom)
                        BestCOPosition.Y = SystemInformation.VirtualScreen.Bottom - (BackgroundSize.Height / 2) - (CentreObjectSize.Height / 2);

                    if (BestCOPosition.X - (BackgroundSize.Width / 2) < SystemInformation.VirtualScreen.Left)
                        BestCOPosition.X = BackgroundSize.Width / 2 - (CentreObjectSize.Width / 2);

                    if (BestCOPosition.X + (BackgroundSize.Width / 2) > SystemInformation.VirtualScreen.Right)
                        BestCOPosition.X = SystemInformation.VirtualScreen.Right - (BackgroundSize.Width / 2) - (CentreObjectSize.Width / 2);

                    CentreObject.Location = BestCOPosition;

                    CalculatedPoints = CalculateDockItemPositions(NewDockItemSizes);
                    SetDockItemPositions(CalculatedPoints);

                    BackgroundObject.Location = new Point(CentreObject.Location.X - (BackgroundSize.Width / 2 - CentreObjectSize.Width / 2),
                        CentreObject.Location.Y - (BackgroundSize.Height / 2 - CentreObjectSize.Height / 2));
                }

                ShowAll();
            }
        }

        public void itemDeactivated()
        {
            if (loaded == false || showingLevel || DockSettings.General.zLevel != "Always On Bottom")
                return;

            IntPtr foregroundWindow = Pinvoke.Win32.GetForegroundWindow();
            try
            {
                if (BackgroundObject.Handle != foregroundWindow && CentreObject.Handle != foregroundWindow)
                {
                    bool dockHasFocus = false;
                    foreach (BaseDockObjects.DockItemObject tempDockItem in MainDockObjects)
                    {
                        if (tempDockItem.Handle == foregroundWindow || tempDockItem.MyLabel.Handle == foregroundWindow)
                        {
                            dockHasFocus = true;
                            break;
                        }
                    }

                    if (dockHasFocus == false)
                        HideAll();
                }
            }
            catch (Exception)
            { }
        }

        public void setZLevels()
        {
            BackgroundObject.SetZLevel();
            CentreObject.SetZLevel();

            foreach (BaseDockObjects.DockItemObject tempDockItem in MainDockObjects)
            {
                tempDockItem.SetZLevel();
                tempDockItem.MyLabel.SetZLevel();
            }
        }

        #endregion

        #region Hotkeys

        private const int ToggleDockHotKeyHandle = 0;
        private const int ModAlt = 1, ModControl = 2, ModShift = 4, ModWin = 8;

        public void UnRegisterHotKeyToggleDock()
        {
            if (ToggleDockHotKeyStatus)
            {
                Pinvoke.Win32.UnregisterHotKey(this.Handle, ToggleDockHotKeyHandle);
            }
        }

        public void RegisterHotKeyToggleDock()
        {
            UnRegisterHotKeyToggleDock();

            // Get numerical representation of the modifier keys
            int modifiers = 0;

            if (DockSettings.Toggling.VisibilityCtrl)
                modifiers = modifiers | ModControl;

            if (DockSettings.Toggling.VisibilityAlt)
                modifiers = modifiers | ModAlt;

            if (DockSettings.Toggling.VisibilityShift)
                modifiers = modifiers | ModShift;

            if (DockSettings.Toggling.VisibilityWin)
                modifiers = modifiers | ModWin;

            // Get numerical representation of the additional keys
            int additonalkeys = 0;

            try
            {
                additonalkeys = additonalkeys | DockSettings.Toggling.VisibilityKey1 | DockSettings.Toggling.VisibilityKey2;
            }
            catch (Exception)
            {
                additonalkeys = 0;
            }

            ToggleDockHotKeyStatus = Pinvoke.Win32.RegisterHotKey(this.Handle, ToggleDockHotKeyHandle, modifiers, additonalkeys);
            //Console.WriteLine(ToggleDockHotKeyStatus + "  " + modifiers + "   " + additonalkeys + "    " + DockSettings.Toggling.VisibilityKey1
            //    + "   " + DockSettings.Toggling.VisibilityKey2);
        }

        #endregion

        #region Global Mouse Hook

        /// <summary>
        /// Starts the mouse hook process. The default hook is for the middle mouse button.
        /// </summary>
        /// <param name="hWnd">Handle to bind the hook to.</param>
        /// <returns></returns>
        [DllImport("Orbit.Hook.dll")]
        public static extern bool StartHook(IntPtr hWnd);

        /// <summary>
        /// Stops the mouse hook process.
        /// </summary>
        /// <param name="hWnd">Handle to unbind the hook from.</param>
        /// <returns></returns>
        [DllImport("Orbit.Hook.dll")]
        public static extern bool StopHook(IntPtr hWnd);

        /// <summary>
        /// Sets the mouse shortcut key.
        /// </summary>
        /// <param name="vKey">Mouse button to hook to.</param>
        [DllImport("Orbit.Hook.dll")]
        public static extern void SetShortcutKey(Keys vKey);

        const int WM_GLOBALMOUSECLICK = 0x0400 + 5000;
        const int WM_GLOBALHOTKEY = 0x0312;

        /// <summary>
        /// Message processing method. Used for our global mouse hook.
        /// </summary>
        /// <param name="m">Window Message</param>
        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {
                case WM_GLOBALMOUSECLICK:
                    ToggleVisiblity();
                    BackgroundObject.Activate();
                    CentreObject.Activate();
                    break;
                case WM_GLOBALHOTKEY:
                    ToggleVisiblity();
                    break;
            }
            base.WndProc(ref m);
        }


        public void MyKeyDown(object sender, KeyEventArgs e)
        {
        }

        public void MyKeyPress(object sender, KeyPressEventArgs e)
        {
        }

        public void MyKeyUp(object sender, KeyEventArgs e)
        {
        }

        public bool InScreenEdgeToggleZone = false;
        public void MouseMoved(object sender, MouseEventArgs e)
        {
            if ((DockSettings.Toggling.ScreenLeftEdgeShow && e.Location.X <= SystemInformation.VirtualScreen.Left + DockSettings.Toggling.EdgeWidthHeight)
                || (DockSettings.Toggling.ScreenTopEdgeShow && e.Location.Y <= SystemInformation.VirtualScreen.Top + DockSettings.Toggling.EdgeWidthHeight)
                || (DockSettings.Toggling.ScreenRightEdgeShow && e.Location.X >= SystemInformation.VirtualScreen.Right - DockSettings.Toggling.EdgeWidthHeight)
                || (DockSettings.Toggling.ScreenBottomEdgeShow && e.Location.Y >= SystemInformation.VirtualScreen.Bottom - DockSettings.Toggling.EdgeWidthHeight))
            {
                if (InScreenEdgeToggleZone == false && MouseButtons == MouseButtons.None)
                {
                    InScreenEdgeToggleZone = true;
                    dwellTime = 0;  
                    mouseTogglingDwellTimer.Start(); 
                }
            }
            else
            {
                InScreenEdgeToggleZone = false;
                mouseTogglingDwellTimer.Stop();
                dwellTime = 0;
            }
        }

        private void mouseTogglingDwellTimer_Tick(object sender, EventArgs e)
        {
            dwellTime += mouseTogglingDwellTimer.Interval;

            if (dwellTime >= DockSettings.Toggling.dwellTime)
            {
                ToggleVisiblity();
                dwellTime = 0;
                mouseTogglingDwellTimer.Stop();
            }
        }

        #endregion

        #region Sorting

        public enum SortDirection
        {
            Ascending,
            Descending
        }

        public class DockItemOrderComparer : IComparer
        {
            private SortDirection m_direction = SortDirection.Ascending;

            public DockItemOrderComparer() : base() { }

            public DockItemOrderComparer(SortDirection direction)
            {
                this.m_direction = direction;
            }

            int IComparer.Compare(object x, object y)
            {
                DockItemObject itemX = (DockItemObject)x;
                DockItemObject itemY = (DockItemObject)y;

                if (itemX == null && itemY == null)
                {
                    return 0;
                }
                else if (itemX == null && itemY != null)
                {
                    return (this.m_direction == SortDirection.Ascending) ? -1 : 1;
                }
                else if (itemX != null && itemY == null)
                {
                    return (this.m_direction == SortDirection.Ascending) ? 1 : -1;
                }
                else
                {
                    int itemXOrder = itemX.dockOrderNum;
                    int itemYOrder = itemY.dockOrderNum;
                    return (this.m_direction == SortDirection.Ascending) ?
                    itemXOrder.CompareTo(itemY.dockOrderNum) :
                    itemYOrder.CompareTo(itemX.dockOrderNum);
                }
            }
        }

        #endregion
    }

}
