﻿// Circle Dock 0.9.2 - Copyright 2008 Eric Wong
// Circle Dock is open source software licensed under GNU GENERAL PUBLIC LICENSE V3. See included Licence.txt file.
// http://circledock.wikidot.com
// VideoInPicture@gmail.com

// This file contains the classes used to load the settings information for the dock using the .ini file format
// to allow for high legibility for the end user.

using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Drawing;
using System.IO;
using System.ComponentModel;

using AMS.Profile;      // Allows for .ini file manipulation
using SettingsInformation;

namespace SettingsLoader
{
    /// <summary>    
    /// Loads the global settings for the dock. Does not include the individual settings for the dock items.
    /// Uses the .ini file format.
    /// </summary>  
    public class SettingsLoader
    {
        #region Variables

        /// <summary>    
        /// Allows for .ini file manipulation.
        /// </summary>  
        private Profile SettingsINI;
        private String DataVerifier;

        public SettingsInformation.Background Background;
        public SettingsInformation.CentreImage CentreImage;
        public SettingsInformation.CircleParams CircleParams;
        public SettingsInformation.DockItems DockItems;
        public SettingsInformation.EllipseParams EllipseParams;
        public SettingsInformation.Language Language;
        public SettingsInformation.ObjectShape Shape;
        public SettingsInformation.Folders Folders;
        public SettingsInformation.General General;
        public SettingsInformation.PoofAnimation PoofAnimation;
        public SettingsInformation.Toggling Toggling;
        public SettingsInformation.Labels Labels;
        public SettingsInformation.GeneralAnimation GeneralAnimation;

        #endregion

        #region Constructor

        /// <summary>        
        /// Loads the global system settings into memory and provides access to the read/write of them.
        /// The reading of the settings should be done using the structs instances declared in this class
        /// for faster access. Reading/writing to the .ini file all the time can lead to low performance.
        /// 
        /// This class is independent of the file version of the .ini file. Bounds checking and automatic repair are done.
        /// </summary>        
        /// <param name="FilePath">System path to the .ini file to where the global settings are stored.</param>        
        public SettingsLoader(String FilePath)
        {
            if (!File.Exists(FilePath))
            {
                MessageBox.Show(@"Config.ini: " + FilePath + @" is missing. Please replace the settings file. Circle Dock will now exit.", "Circle Dock");
                Application.Exit();
            }

            SettingsINI = new Ini(FilePath);
            InitializeSettingsInfoStructs();
            SettingsLoaderLoadData();
        }

        #endregion

        #region Methods

        /// <summary>        
        /// Initializes the structs used to store the settings data. Can be used to clear the settings from memory.
        /// </summary>        
        public void InitializeSettingsInfoStructs()
        {
            Background = new SettingsInformation.Background();
            CentreImage = new SettingsInformation.CentreImage();
            CircleParams = new SettingsInformation.CircleParams();
            DockItems = new SettingsInformation.DockItems();
            EllipseParams = new SettingsInformation.EllipseParams();
            Language = new SettingsInformation.Language();
            Shape = new SettingsInformation.ObjectShape();
            Folders = new SettingsInformation.Folders();
            General = new SettingsInformation.General();
            PoofAnimation = new SettingsInformation.PoofAnimation();
            Toggling = new SettingsInformation.Toggling();
            Labels = new SettingsInformation.Labels();
            GeneralAnimation = new SettingsInformation.GeneralAnimation();
        }

        /// <summary>        
        /// Loads the global settings from the .ini file into the structs of this class. Can be used to refresh the settings
        /// data from the settings file into memory.
        /// </summary>  
        public void SettingsLoaderLoadData()
        {
            LoadBackground();
            LoadCentreImage();
            LoadCircleParams();
            LoadDockItems();
            LoadEllipseParams();
            LoadLanguage();
            LoadShape();
            LoadFolders();
            LoadGeneral();
            LoadPoofAnimation();
            LoadToggling();
            LoadLabels();
            LoadGeneralAnimation();
        }

        /// <summary>        
        /// Retrieves the setting value at the given Section with the name of EntryName.
        /// </summary>        
        /// <param name="Section">The section in the .ini file to look at.</param>        
        /// <returns>A string representing the setting value or null if not found.</returns> 
        public String GetEntry(String Section, String EntryName)
        {
            return (String)SettingsINI.GetValue(Section, EntryName);
        }

        /// <summary>        
        /// Retrieves all the section names in the settings file.
        /// </summary>               
        /// <returns>A string array containing the section names null if not found.</returns> 
        public String[] GetSectionNames()
        {
            return SettingsINI.GetSectionNames();
        }

        /// <summary>        
        /// Retrieves the all the names of the settings entries under the given Section
        /// </summary>        
        /// <param name="Section">The section in the .ini file to look at.</param>        
        /// <returns>A string array containing all the entry names in the given section or null if not found.</returns> 
        public String[] GetEntryNames(String Section)
        {
            return SettingsINI.GetEntryNames(Section);
        }

        /// <summary>        
        /// Sets the value of a setting.
        /// </summary>        
        /// <param name="Section">The section in the .ini file to write to. It is created if not found.</param>  
        /// <param name="EntryName">The entry name to store the setting value under. It is created if not found.</param>
        /// <param name="Value">A string representing the value to be stored.</param>
        public void SetEntry(String Section, String EntryName, String Value)
        {
            SettingsINI.SetValue(Section, EntryName, Value);
        }

        /// <summary>        
        /// Removes an entry from the settings file.
        /// </summary>        
        /// <param name="Section">The section in the .ini file to search in.</param>  
        /// <param name="EntryName">The entry to remove.</param>
        public void RemoveEntry(String Section, String EntryName)
        {
            SettingsINI.RemoveEntry(Section, EntryName);
        }

        /// <summary>        
        /// Removes an entire section from the settings file, including the entries.
        /// </summary>        
        /// <param name="Section">The section in the .ini file to remove.</param>  
        public void RemoveSection(String Section)
        {
            SettingsINI.RemoveSection(Section);
        }

        #endregion

        #region Load Each Section for LanguageInformation

        private void LoadBackground()
        {
            const String theSection = "Background";

            #region Path

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "Path");

            if (DataVerifier == null)
            {
                DataVerifier = @"";
                SetEntry(theSection, "Path", DataVerifier);
            }

            Background.Path = DataVerifier;

            #endregion

            #region DefaultOpacity

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultOpacity");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "DefaultOpacity", DataVerifier);
            }

            try
            {
                Background.DefaultOpacity = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Background.DefaultOpacity = 255;
                SetEntry(theSection, "DefaultOpacity", @"255");
            }

            if (Background.DefaultOpacity < 0)
            {
                Background.DefaultOpacity = 0;
                SetEntry(theSection, "DefaultOpacity", @"0");
            }
            else if (Background.DefaultOpacity > 255)
            {
                Background.DefaultOpacity = 255;
                SetEntry(theSection, "DefaultOpacity", @"255");
            }

            #endregion

            #region DefaultWidth

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultWidth");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "DefaultWidth", DataVerifier);
            }

            try
            {
                Background.DefaultWidth = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Background.DefaultWidth = 255;
                SetEntry(theSection, "DefaultWidth", @"255");
            }

            if (Background.DefaultWidth <= 0)
            {
                Background.DefaultWidth = 255;
                SetEntry(theSection, "DefaultWidth", @"255");
            }

            #endregion

            #region DefaultHeight

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultHeight");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "DefaultHeight", DataVerifier);
            }

            try
            {
                Background.DefaultHeight = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Background.DefaultHeight = 255;
                SetEntry(theSection, "DefaultHeight", @"255");
            }

            if (Background.DefaultHeight <= 0)
            {
                Background.DefaultHeight = 255;
                SetEntry(theSection, "DefaultHeight", @"255");
            }

            #endregion
        }

        private void LoadCentreImage()
        {
            const String theSection = "CentreImage";

            #region Path

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "Path");

            if (DataVerifier == null)
            {
                DataVerifier = @"";
                SetEntry(theSection, "Path", DataVerifier);
            }

            CentreImage.Path = DataVerifier;

            #endregion

            #region DefaultOpacity

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultOpacity");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "DefaultOpacity", DataVerifier);
            }

            try
            {
                CentreImage.DefaultOpacity = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                CentreImage.DefaultOpacity = 255;
                SetEntry(theSection, "DefaultOpacity", @"255");
            }

            if (CentreImage.DefaultOpacity < 0)
            {
                CentreImage.DefaultOpacity = 0;
                SetEntry(theSection, "DefaultOpacity", @"0");
            }
            else if (CentreImage.DefaultOpacity > 255)
            {
                CentreImage.DefaultOpacity = 255;
                SetEntry(theSection, "DefaultOpacity", @"255");
            }

            #endregion

            #region DefaultWidth

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultWidth");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "DefaultWidth", DataVerifier);
            }

            try
            {
                CentreImage.DefaultWidth = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                CentreImage.DefaultWidth = 255;
                SetEntry(theSection, "DefaultWidth", @"255");
            }

            if (CentreImage.DefaultWidth <= 0)
            {
                CentreImage.DefaultWidth = 255;
                SetEntry(theSection, "DefaultWidth", @"255");
            }

            #endregion

            #region DefaultHeight

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultHeight");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "DefaultHeight", DataVerifier);
            }

            try
            {
                CentreImage.DefaultHeight = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                CentreImage.DefaultHeight = 255;
                SetEntry(theSection, "DefaultHeight", @"255");
            }

            if (CentreImage.DefaultHeight <= 0)
            {
                CentreImage.DefaultHeight = 255;
                SetEntry(theSection, "DefaultHeight", @"255");
            }

            #endregion

            #region ShowStartMenuWhenClicked

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ShowStartMenuWhenClicked");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "ShowStartMenuWhenClicked", DataVerifier);
            }

            try
            {
                CentreImage.ShowStartMenuWhenClicked = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                CentreImage.ShowStartMenuWhenClicked = false;
                SetEntry(theSection, "ShowStartMenuWhenClicked", false.ToString());
            }

            #endregion
        }

        private void LoadCircleParams()
        {
            const String theSection = "CircleParams";

            #region CircleSeparation

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "CircleSeparation");

            if (DataVerifier == null)
            {
                DataVerifier = @"155";
                SetEntry(theSection, "CircleSeparation", DataVerifier);
            }

            try
            {
                CircleParams.CircleSeparation = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                CircleParams.CircleSeparation = 155;
                SetEntry(theSection, "CircleSeparation", @"155");
            }

            if (CircleParams.CircleSeparation < 0)
            {
                CircleParams.CircleSeparation = 155;
                SetEntry(theSection, "CircleSeparation", @"155");
            }

            #endregion

            #region MinRadius

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "MinRadius");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "MinRadius", DataVerifier);
            }

            try
            {
                CircleParams.MinRadius = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                CircleParams.MinRadius = 255;
                SetEntry(theSection, "MinRadius", @"255");
            }

            if (CircleParams.MinRadius < 0)
            {
                CircleParams.MinRadius = 255;
                SetEntry(theSection, "MinRadius", @"255");
            }

            #endregion

            #region Format

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "Format");

            ArrayList ValidValues = new ArrayList();
            ValidValues.Add(@"Constant Number of Items Per Circle");
            ValidValues.Add(@"Maximum Number of Items Per Circle");

            if (DataVerifier == null || ValidValues.Contains(DataVerifier) == false)
            {
                DataVerifier = @"Constant Number of Items Per Circle";
                SetEntry(theSection, "Format", DataVerifier);
            }

            CircleParams.Format = DataVerifier;

            #endregion

            #region ConstNumItemsPerCircle

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ConstNumItemsPerCircle");

            if (DataVerifier == null)
            {
                DataVerifier = @"10";
                SetEntry(theSection, "ConstNumItemsPerCircle", DataVerifier);
            }

            try
            {
                CircleParams.ConstNumItemsPerCircle = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                CircleParams.ConstNumItemsPerCircle = 10;
                SetEntry(theSection, "ConstNumItemsPerCircle", @"10");
            }

            if (CircleParams.ConstNumItemsPerCircle < 0)
            {
                CircleParams.ConstNumItemsPerCircle = 10;
                SetEntry(theSection, "ConstNumItemsPerCircle", @"10");
            }

            #endregion
        }

        private void LoadDockItems()
        {
            const String theSection = "DockItems";

            #region DefaultHeight

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultHeight");

            if (DataVerifier == null)
            {
                DataVerifier = @"60";
                SetEntry(theSection, "DefaultHeight", DataVerifier);
            }

            try
            {
                DockItems.DefaultHeight = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.DefaultHeight = 60;
                SetEntry(theSection, "DefaultHeight", @"60");
            }

            if (DockItems.DefaultHeight <= 0)
            {
                DockItems.DefaultHeight = 60;
                SetEntry(theSection, "DefaultHeight", @"60");
            }

            #endregion

            #region DefaultWidth

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultWidth");

            if (DataVerifier == null)
            {
                DataVerifier = @"60";
                SetEntry(theSection, "DefaultWidth", DataVerifier);
            }

            try
            {
                DockItems.DefaultWidth = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.DefaultWidth = 60;
                SetEntry(theSection, "DefaultWidth", @"60");
            }

            if (DockItems.DefaultWidth <= 0)
            {
                DockItems.DefaultWidth = 60;
                SetEntry(theSection, "DefaultWidth", @"60");
            }

            #endregion

            #region MaxHeight

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "MaxHeight");

            if (DataVerifier == null)
            {
                DataVerifier = @"100";
                SetEntry(theSection, "MaxHeight", DataVerifier);
            }

            try
            {
                DockItems.MaxHeight = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.MaxHeight = 100;
                SetEntry(theSection, "MaxHeight", @"100");
            }

            if (DockItems.MaxHeight <= 0)
            {
                DockItems.MaxHeight = 100;
                SetEntry(theSection, "MaxHeight", @"100");
            }

            #endregion

            #region MaxWidth

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "MaxWidth");

            if (DataVerifier == null)
            {
                DataVerifier = @"100";
                SetEntry(theSection, "MaxWidth", DataVerifier);
            }

            try
            {
                DockItems.MaxWidth = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.MaxWidth = 100;
                SetEntry(theSection, "MaxWidth", @"100");
            }

            if (DockItems.MaxWidth <= 0)
            {
                DockItems.MaxWidth = 100;
                SetEntry(theSection, "MaxWidth", @"100");
            }

            #endregion

            #region MinHeight

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "MinHeight");

            if (DataVerifier == null)
            {
                DataVerifier = @"40";
                SetEntry(theSection, "MinHeight", DataVerifier);
            }

            try
            {
                DockItems.MinHeight = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.MinHeight = 40;
                SetEntry(theSection, "MinHeight", @"40");
            }

            if (DockItems.MinHeight <= 0)
            {
                DockItems.MinHeight = 40;
                SetEntry(theSection, "MinHeight", @"40");
            }

            #endregion

            #region MinWidth

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "MinWidth");

            if (DataVerifier == null)
            {
                DataVerifier = @"40";
                SetEntry(theSection, "MinWidth", DataVerifier);
            }

            try
            {
                DockItems.MinWidth = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.MinWidth = 40;
                SetEntry(theSection, "MinWidth", @"40");
            }

            if (DockItems.MinWidth <= 0)
            {
                DockItems.MinWidth = 40;
                SetEntry(theSection, "MinWidth", @"40");
            }

            #endregion

            #region ShowExplorerContextMenus

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ShowExplorerContextMenus");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "ShowExplorerContextMenus", DataVerifier);
            }

            try
            {
                DockItems.ShowExplorerContextMenus = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.ShowExplorerContextMenus = false;
                SetEntry(theSection, "ShowExplorerContextMenus", false.ToString());
            }

            #endregion

            #region HideDockAfterExecution

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "HideDockAfterExecution");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "HideDockAfterExecution", DataVerifier);
            }

            try
            {
                DockItems.HideDockAfterExecution = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.HideDockAfterExecution = false;
                SetEntry(theSection, "HideDockAfterExecution", false.ToString());
            }

            #endregion

            #region DefaultOpacity

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultOpacity");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "DefaultOpacity", DataVerifier);
            }

            try
            {
                DockItems.DefaultOpacity = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DockItems.DefaultOpacity = 255;
                SetEntry(theSection, "DefaultOpacity", @"255");
            }

            if (DockItems.DefaultOpacity < 0)
            {
                DockItems.DefaultOpacity = 0;
                SetEntry(theSection, "DefaultOpacity", @"0");
            }
            else if (DockItems.DefaultOpacity > 255)
            {
                DockItems.DefaultOpacity = 255;
                SetEntry(theSection, "DefaultOpacity", @"255");
            }

            #endregion
        }

        private void LoadEllipseParams()
        {
            const String theSection = "EllipseParams";

            #region MinHeight

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "MinHeight");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "MinHeight", DataVerifier);
            }

            try
            {
                EllipseParams.MinHeight = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                EllipseParams.MinHeight = 255;
                SetEntry(theSection, "MinHeight", @"255");
            }

            if (EllipseParams.MinHeight <= 0)
            {
                EllipseParams.MinHeight = 255;
                SetEntry(theSection, "MinHeight", @"255");
            }

            #endregion

            #region AspectRatio

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "AspectRatio");

            if (DataVerifier == null)
            {
                DataVerifier = @"1";
                SetEntry(theSection, "AspectRatio", DataVerifier);
            }

            try
            {
                EllipseParams.AspectRatio = double.Parse(DataVerifier);
            }
            catch (Exception)
            {
                EllipseParams.AspectRatio = 1;
                SetEntry(theSection, "AspectRatio", @"1");
            }

            if (EllipseParams.AspectRatio <= 0)
            {
                EllipseParams.AspectRatio = 1;
                SetEntry(theSection, "AspectRatio", @"1");
            }

            #endregion
        }

        private void LoadLanguage()
        {
            const String theSection = "Language";

            #region Path

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "Path");

            if (DataVerifier == null)
            {
                DataVerifier = @"";
                SetEntry(theSection, "Path", DataVerifier);
            }

            Language.path = DataVerifier;

            #endregion
        }

        private void LoadShape()
        {
            const String theSection = "Shape";

            #region Shape

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "Shape");

            ArrayList ValidValues = new ArrayList();
            ValidValues.Add("Circle");

            if (DataVerifier == null || ValidValues.Contains(DataVerifier) == false)
            {
                DataVerifier = @"Circle";
                SetEntry(theSection, "Shape", DataVerifier);
            }

            Shape.Shape = DataVerifier;

            #endregion
        }

        private void LoadFolders()
        {
            const String theSection = "Folders";

            #region Path

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DockFolderImagePath");

            if (DataVerifier == null)
            {
                DataVerifier = @"";
                SetEntry(theSection, "DockFolderImagePath", DataVerifier);
            }

            Folders.DockFolderImagePath = DataVerifier;

            #endregion
        }

        private void LoadGeneral()
        {
            const String theSection = "General";

            #region ShowLevelRealTimeCreation

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ShowLevelRealTimeCreation");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "ShowLevelRealTimeCreation", DataVerifier);
            }

            try
            {
                General.ShowLevelRealTimeCreation = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.ShowLevelRealTimeCreation = false;
                SetEntry(theSection, "ShowLevelRealTimeCreation", false.ToString());
            }

            #endregion

            #region CentreAroundCursor

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "CentreAroundCursor");

            if (DataVerifier == null)
            {
                DataVerifier = true.ToString();
                SetEntry(theSection, "CentreAroundCursor", DataVerifier);
            }

            try
            {
                General.CentreAroundCursor = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.CentreAroundCursor = true;
                SetEntry(theSection, "CentreAroundCursor", true.ToString());
            }

            #endregion

            #region EnableDockRotation

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "EnableDockRotation");

            if (DataVerifier == null)
            {
                DataVerifier = true.ToString();
                SetEntry(theSection, "EnableDockRotation", DataVerifier);
            }

            try
            {
                General.EnableDockRotation = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.EnableDockRotation = true;
                SetEntry(theSection, "EnableDockRotation", true.ToString());
            }

            #endregion

            #region AnimationInterval

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "AnimationInterval");

            if (DataVerifier == null)
            {
                DataVerifier = @"20";
                SetEntry(theSection, "AnimationInterval", DataVerifier);
            }

            try
            {
                General.AnimationInterval = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.AnimationInterval = 20;
                SetEntry(theSection, "AnimationInterval", @"20");
            }

            if (General.AnimationInterval <= 0)
            {
                General.AnimationInterval = 20;
                SetEntry(theSection, "AnimationInterval", @"20");
            }

            #endregion

            #region ClickSensitivity

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ClickSensitivity");

            if (DataVerifier == null)
            {
                DataVerifier = @"0";
                SetEntry(theSection, "ClickSensitivity", DataVerifier);
            }

            try
            {
                General.ClickSensitivity = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.ClickSensitivity = 0;
                SetEntry(theSection, "ClickSensitivity", @"0");
            }

            if (General.ClickSensitivity < 0)
            {
                General.ClickSensitivity = 0;
                SetEntry(theSection, "ClickSensitivity", @"0");
            }
            else if (General.ClickSensitivity > 100)
            {
                General.ClickSensitivity = 100;
                SetEntry(theSection, "ClickSensitivity", @"100");
            }

            #endregion

            #region EnablePortabilityMode

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "EnablePortabilityMode");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "EnablePortabilityMode", DataVerifier);
            }

            try
            {
                General.EnablePortabilityMode = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.EnablePortabilityMode = false;
                SetEntry(theSection, "EnablePortabilityMode", false.ToString());
            }

            #endregion

            #region KeyPressesPerRevolution

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "KeyPressesPerRevolution");

            if (DataVerifier == null)
            {
                DataVerifier = @"15";
                SetEntry(theSection, "KeyPressesPerRevolution", DataVerifier);
            }

            try
            {
                General.KeyPressesPerRevolution = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.KeyPressesPerRevolution = 15;
                SetEntry(theSection, "KeyPressesPerRevolution", @"15");
            }

            if (General.KeyPressesPerRevolution < 0)
            {
                General.KeyPressesPerRevolution = 15;
                SetEntry(theSection, "KeyPressesPerRevolution", @"15");
            }
            else if (General.KeyPressesPerRevolution > 50)
            {
                General.KeyPressesPerRevolution = 15;
                SetEntry(theSection, "KeyPressesPerRevolution", @"50");
            }

            #endregion

            #region UseMemorySaver

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "UseMemorySaver");

            if (DataVerifier == null)
            {
                DataVerifier = true.ToString();
                SetEntry(theSection, "UseMemorySaver", DataVerifier);
            }

            try
            {
                General.UseMemorySaver = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.UseMemorySaver = true;
                SetEntry(theSection, "UseMemorySaver", true.ToString());
            }

            #endregion

            #region ShowDockOnLoad

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "showDockOnLoad");

            if (DataVerifier == null)
            {
                DataVerifier = true.ToString();
                SetEntry(theSection, "showDockOnLoad", DataVerifier);
            }

            try
            {
                General.ShowDock = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.UseMemorySaver = true;
                SetEntry(theSection, "showDockOnLoad", true.ToString());
            }

            #endregion

            #region AlwaysShowTopLevel

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "alwaysShowTopLevel");

            if (DataVerifier == null)
            {
                DataVerifier = true.ToString();
                SetEntry(theSection, "alwaysShowTopLevel", DataVerifier);
            }

            try
            {
                General.ShowDock = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.AlwaysShowTopLevel = false;
                SetEntry(theSection, "alwaysShowTopLevel", false.ToString());
            }

            #endregion

            #region IconReplacementMode

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "IconReplacementMode");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "IconReplacementMode", DataVerifier);
            }

            try
            {
                General.IconReplacementMode = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.IconReplacementMode = false;
                SetEntry(theSection, "IconReplacementMode", false.ToString());
            }

            #endregion

            #region Z-Level

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "zLevel");

            if (DataVerifier == null || (DataVerifier != @"Topmost" && DataVerifier != @"Normal" && DataVerifier != "Always On Bottom"))
            {
                DataVerifier = "Topmost";
                SetEntry(theSection, "zLevel", DataVerifier);
            }

            General.zLevel = DataVerifier;

            #endregion

            #region lockDockPosition

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "lockDockPosition");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "lockDockPosition", DataVerifier);
            }

            try
            {
                General.lockDockPosition = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.lockDockPosition = false;
                SetEntry(theSection, "lockDockPosition", false.ToString());
            }

            #endregion

            #region lockedPosition

            TypeConverter PointConverter = TypeDescriptor.GetConverter(typeof(Point));

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "lockedPosition");

            if (DataVerifier == null)
            {
                DataVerifier = PointConverter.ConvertToString(new Point(0, 0));
                SetEntry(theSection, "lockedPosition", DataVerifier);
            }

            try
            {
                General.lockedPosition = (Point)PointConverter.ConvertFromString(DataVerifier);
            }
            catch (Exception)
            {
                General.lockedPosition = new Point(0, 0);
                SetEntry(theSection, "lockedPosition", PointConverter.ConvertToString(new Point(0, 0)));
            }

            #endregion

            #region useSameRotationValues

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "useSameRotationValues");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "useSameRotationValues", DataVerifier);
            }

            try
            {
                General.useSameRotationValues = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.useSameRotationValues = false;
                SetEntry(theSection, "useSameRotationValues", false.ToString());
            }

            #endregion

            #region ShowDock
            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "showDockOnLoad");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "showDockOnLoad", DataVerifier);
            }

            try
            {
                General.ShowDock = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.ShowDock = false;
                SetEntry(theSection, "showDockOnLoad", false.ToString());
            }

            #endregion

            #region AlwaysShowTopLevel
            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "alwaysShowTopLevel");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "alwaysShowTopLevel", DataVerifier);
            }

            try
            {
                General.AlwaysShowTopLevel = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                General.AlwaysShowTopLevel = false;
                SetEntry(theSection, "alwaysShowTopLevel", false.ToString());
            }


            #endregion
        }

        private void LoadPoofAnimation()
        {
            const String theSection = "PoofAnimation";

            #region PoofAnimationPath

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "Path");

            if (DataVerifier == null)
            {
                DataVerifier = @"";
                SetEntry(theSection, "Path", DataVerifier);
            }

            PoofAnimation.PoofAnimationPath = DataVerifier;

            #endregion

            #region NumFrames
            //Need to finish implementing poof animation;

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "NumFrames");

            ArrayList ValidValues = new ArrayList();
            ValidValues.Add(6);  

            if (DataVerifier == null || ValidValues.Contains(DataVerifier) == false)
            {
                DataVerifier = @"6";
                SetEntry(theSection, "NumFrames", DataVerifier);
            }

            try
            {
                //PoofAnimation.NumFrames = int.Parse(DataVerifier);
                PoofAnimation.NumFrames = 6;
            }
            catch (Exception)
            {
                PoofAnimation.NumFrames = 6;
            }

            #endregion

            #region DelayBetweenFrames

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DelayBetweenFrames");

            if (DataVerifier == null)
            {
                DataVerifier = @"50";
                SetEntry(theSection, "DelayBetweenFrames", DataVerifier);
            }

            try
            {
                PoofAnimation.DelayBetweenFrames = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                PoofAnimation.DelayBetweenFrames = 50;
                SetEntry(theSection, "DelayBetweenFrames", @"50");
            }

            if (PoofAnimation.DelayBetweenFrames <= 0)
            {
                PoofAnimation.DelayBetweenFrames = 50;
                SetEntry(theSection, "DelayBetweenFrames", @"50");
            }

            #endregion

            #region UsePoof

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "UsePoof");

            if (DataVerifier == null)
            {
                DataVerifier = true.ToString();
                SetEntry(theSection, "UsePoof", DataVerifier);
            }

            try
            {
                PoofAnimation.UsePoof = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                PoofAnimation.UsePoof = true;
                SetEntry(theSection, "UsePoof", true.ToString());
            }

            #endregion
        }

        private void LoadToggling()
        {
            const String theSection = "Toggling";

            KeysConverter ConvertKeys = new KeysConverter();

            #region VisibilityCtrl

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "VisibilityCtrl");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "VisibilityCtrl", DataVerifier);
            }

            try
            {
                Toggling.VisibilityCtrl = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.VisibilityCtrl = false;
                SetEntry(theSection, "VisibilityCtrl", false.ToString());
            }

            #endregion

            #region VisibilityAlt

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "VisibilityAlt");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "VisibilityAlt", DataVerifier);
            }

            try
            {
                Toggling.VisibilityAlt = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.VisibilityAlt = false;
                SetEntry(theSection, "VisibilityAlt", false.ToString());
            }

            #endregion

            #region VisibilityShift

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "VisibilityShift");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "VisibilityShift", DataVerifier);
            }

            try
            {
                Toggling.VisibilityShift = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.VisibilityShift = false;
                SetEntry(theSection, "VisibilityShift", false.ToString());
            }

            #endregion

            #region VisibilityWin

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "VisibilityWin");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "VisibilityWin", DataVerifier);
            }

            try
            {
                Toggling.VisibilityWin = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.VisibilityWin = false;
                SetEntry(theSection, "VisibilityWin", false.ToString());
            }

            #endregion

            #region VisibilityKey1

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "VisibilityKey1");

            if (DataVerifier == null)
            {
                DataVerifier = ConvertKeys.ConvertToString(Keys.F1);
                SetEntry(theSection, "VisibilityKey1", DataVerifier);
            }

            try
            {
                Toggling.VisibilityKey1 = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DataVerifier = ConvertKeys.ConvertToString(Keys.F1);
                SetEntry(theSection, "VisibilityKey1", DataVerifier);
            }

            #endregion

            #region VisibilityKey2

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "VisibilityKey2");

            if (DataVerifier == null)
            {
                DataVerifier = ConvertKeys.ConvertToString(Keys.None);
                SetEntry(theSection, "VisibilityKey2", DataVerifier);
            }

            try
            {
                Toggling.VisibilityKey2 = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                DataVerifier = ConvertKeys.ConvertToString(Keys.None);
                SetEntry(theSection, "VisibilityKey2", DataVerifier);
            }

            #endregion

            #region VisibilityMouseButton

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "VisibilityMouseButton");

            ArrayList ValidValues = new ArrayList();
            ValidValues.Add(ConvertKeys.ConvertToString(Keys.None));
            ValidValues.Add(ConvertKeys.ConvertToString(Keys.MButton));
            ValidValues.Add(ConvertKeys.ConvertToString(Keys.XButton1));
            ValidValues.Add(ConvertKeys.ConvertToString(Keys.XButton2));

            if (DataVerifier == null || ValidValues.Contains(DataVerifier) == false)
            {
                DataVerifier = ConvertKeys.ConvertToString(Keys.MButton);
                SetEntry(theSection, "VisibilityMouseButton", DataVerifier);
            }

            try
            {
                Toggling.VisibilityMouseButton = ConvertKeys.ConvertToString(DataVerifier);
            }
            catch (Exception)
            {
                DataVerifier = ConvertKeys.ConvertToString(Keys.MButton);
                SetEntry(theSection, "VisibilityMouseButton", DataVerifier);
            }

            #endregion

            #region ScreenLeftEdgeShow

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ScreenLeftEdgeShow");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "ScreenLeftEdgeShow", DataVerifier);
            }

            try
            {
                Toggling.ScreenLeftEdgeShow = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.ScreenLeftEdgeShow = false;
                SetEntry(theSection, "ScreenLeftEdgeShow", false.ToString());
            }

            #endregion

            #region ScreenRightEdgeShow

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ScreenRightEdgeShow");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "ScreenRightEdgeShow", DataVerifier);
            }

            try
            {
                Toggling.ScreenRightEdgeShow = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.ScreenRightEdgeShow = false;
                SetEntry(theSection, "ScreenRightEdgeShow", false.ToString());
            }

            #endregion

            #region ScreenTopEdgeShow

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ScreenTopEdgeShow");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "ScreenTopEdgeShow", DataVerifier);
            }

            try
            {
                Toggling.ScreenTopEdgeShow = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.ScreenTopEdgeShow = false;
                SetEntry(theSection, "ScreenTopEdgeShow", false.ToString());
            }

            #endregion

            #region ScreenBottomEdgeShow

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ScreenBottomEdgeShow");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "ScreenBottomEdgeShow", DataVerifier);
            }

            try
            {
                Toggling.ScreenBottomEdgeShow = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.ScreenBottomEdgeShow = false;
                SetEntry(theSection, "ScreenBottomEdgeShow", false.ToString());
            }

            #endregion

            #region AutoHide

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "AutoHideDock");

            if (DataVerifier == null)
            {
                DataVerifier = false.ToString();
                SetEntry(theSection, "AutoHideDock", DataVerifier);
            }

            try
            {
                Toggling.AutoHide = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.AutoHide = false;
                SetEntry(theSection, "AutoHideDock", false.ToString());
            }

            #endregion


            #region EdgeWidthHeight

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "EdgeWidthHeight");

            if (DataVerifier == null)
            {
                DataVerifier = @"1";
                SetEntry(theSection, "EdgeWidthHeight", DataVerifier);
            }

            try
            {
                Toggling.EdgeWidthHeight = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.EdgeWidthHeight = 1;
                SetEntry(theSection, "EdgeWidthHeight", @"1");
            }

            if (Toggling.EdgeWidthHeight < 1)
            {
                Toggling.EdgeWidthHeight = 1;
                SetEntry(theSection, "EdgeWidthHeight", @"1");
            }
            else if (Toggling.EdgeWidthHeight > 10)
            {
                Toggling.EdgeWidthHeight = 10;
                SetEntry(theSection, "EdgeWidthHeight", @"10");
            }

            #endregion

            #region dwellTime

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "dwellTime");

            if (DataVerifier == null)
            {
                DataVerifier = @"200";
                SetEntry(theSection, "dwellTime", DataVerifier);
            }

            try
            {
                Toggling.dwellTime = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Toggling.dwellTime = 200;
                SetEntry(theSection, "dwellTime", @"200");
            }

            if (Toggling.dwellTime < 0)
            {
                Toggling.dwellTime = 0;
                SetEntry(theSection, "dwellTime", @"0");
            }
            else if (Toggling.dwellTime > 1000)
            {
                Toggling.dwellTime = 1000;
                SetEntry(theSection, "dwellTime", @"1000");
            }

            #endregion
        }

        private void LoadLabels()
        {
            const String theSection = "Labels";

            TypeConverter FontConverter = TypeDescriptor.GetConverter(typeof(Font));
            TypeConverter ColorConverter = TypeDescriptor.GetConverter(typeof(Color));

            #region FontName

            try
            {
                Labels.FontName = (Font)FontConverter.ConvertFromString(GetEntry(theSection, "FontName"));

                if (Labels.FontName == null)
                {
                    Labels.FontName = new Font(System.Drawing.FontFamily.GenericSansSerif, 16, FontStyle.Bold);
                    SetEntry(theSection, "FontName", FontConverter.ConvertToString(Labels.FontName));
                }
            }
            catch (Exception)
            {
                Labels.FontName = new Font(System.Drawing.FontFamily.GenericSansSerif, 16, FontStyle.Bold);
                SetEntry(theSection, "FontName", FontConverter.ConvertToString(Labels.FontName));
            }

            #endregion

            #region FontColor

            try
            {
                Labels.FontColor = Color.FromArgb(int.Parse(GetEntry(theSection, "FontColor")));
                if (Labels.FontColor == null)
                {
                    Labels.FontColor = Color.Black;
                    SetEntry(theSection, "FontColor", Labels.FontColor.ToArgb().ToString());
                }
            }
            catch (Exception)
            {
                Labels.FontColor = Color.Black;
                SetEntry(theSection, "FontColor", Labels.FontColor.ToArgb().ToString());
            }

            #endregion

            #region ShadowColor

            try
            {
                Labels.ShadowColor = Color.FromArgb(int.Parse(GetEntry(theSection, "ShadowColor")));
                if (Labels.ShadowColor == null)
                {
                    Labels.ShadowColor = Color.White;
                    SetEntry(theSection, "ShadowColor", Labels.ShadowColor.ToArgb().ToString());
                }
            }
            catch (Exception)
            {
                Labels.ShadowColor = Color.White;
                SetEntry(theSection, "ShadowColor", Labels.ShadowColor.ToArgb().ToString());
            }

            #endregion

            #region ShadowSize

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ShadowSize");

            if (DataVerifier == null)
            {
                DataVerifier = @"6";
                SetEntry(theSection, "ShadowSize", DataVerifier);
            }

            try
            {
                Labels.ShadowSize = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Labels.ShadowSize = 6;
                SetEntry(theSection, "ShadowSize", @"6");
            }

            if (Labels.ShadowSize < 0)
            {
                Labels.ShadowSize = 0;
                SetEntry(theSection, "ShadowSize", @"0");
            }
            else if (Labels.ShadowSize > 15)
            {
                Labels.ShadowSize = 15;
                SetEntry(theSection, "ShadowSize", @"15");
            }

            #endregion

            #region ShadowDarkness

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ShadowDarkness");

            if (DataVerifier == null)
            {
                DataVerifier = @"10";
                SetEntry(theSection, "ShadowDarkness", DataVerifier);
            }

            try
            {
                Labels.ShadowDarkness = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Labels.ShadowDarkness = 10;
                SetEntry(theSection, "ShadowDarkness", @"10");
            }

            if (Labels.ShadowDarkness < 0)
            {
                Labels.ShadowDarkness = 0;
                SetEntry(theSection, "ShadowDarkness", @"0");
            }
            else if (Labels.ShadowDarkness > 25)
            {
                Labels.ShadowDarkness = 25;
                SetEntry(theSection, "ShadowDarkness", @"25");
            }

            #endregion

            #region ShowLabels

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ShowLabels");

            if (DataVerifier == null)
            {
                DataVerifier = true.ToString();
                SetEntry(theSection, "ShowLabels", DataVerifier);
            }

            try
            {
                Labels.ShowLabels = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Labels.ShowLabels = true;
                SetEntry(theSection, "ShowLabels", true.ToString());
            }

            #endregion

            #region ShowLabelsOnMouseOverOnly

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "ShowLabelsOnMouseOverOnly");

            if (DataVerifier == null)
            {
                DataVerifier = true.ToString();
                SetEntry(theSection, "ShowLabelsOnMouseOverOnly", DataVerifier);
            }

            try
            {
                Labels.ShowLabelsOnMouseOverOnly = bool.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Labels.ShowLabelsOnMouseOverOnly = true;
                SetEntry(theSection, "ShowLabelsOnMouseOverOnly", true.ToString());
            }

            #endregion

            #region DefaultOpacity

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "DefaultOpacity");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "DefaultOpacity", DataVerifier);
            }

            try
            {
                Labels.DefaultOpacity = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                Labels.DefaultOpacity = 255;
                SetEntry(theSection, "DefaultOpacity", @"255");
            }

            if (Labels.DefaultOpacity < 0)
            {
                Labels.DefaultOpacity = 0;
                SetEntry(theSection, "DefaultOpacity", @"0");
            }
            else if (Labels.DefaultOpacity > 255)
            {
                Labels.DefaultOpacity = 255;
                SetEntry(theSection, "DefaultOpacity", @"255");
            }

            #endregion
        }

        private void LoadGeneralAnimation()
        {
            const String theSection = "GeneralAnimation";

            #region RotationAnimationDuration

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "RotationAnimationDuration");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "RotationAnimationDuration", DataVerifier);
            }

            try
            {
                GeneralAnimation.RotationAnimationDuration = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                GeneralAnimation.RotationAnimationDuration = 255;
            }

            if (GeneralAnimation.RotationAnimationDuration <= 0)
                GeneralAnimation.RotationAnimationDuration = 255;

            #endregion

            #region FadeInFadeOutDuration

            DataVerifier = null;
            DataVerifier = GetEntry(theSection, "FadeInFadeOutDuration");

            if (DataVerifier == null)
            {
                DataVerifier = @"255";
                SetEntry(theSection, "FadeInFadeOutDuration", DataVerifier);
            }

            try
            {
                GeneralAnimation.FadeInFadeOutDuration = int.Parse(DataVerifier);
            }
            catch (Exception)
            {
                GeneralAnimation.FadeInFadeOutDuration = 255;
            }

            if (GeneralAnimation.FadeInFadeOutDuration <= 0)
                GeneralAnimation.FadeInFadeOutDuration = 255;

            #endregion
        }

        #endregion
    }
}
